require 'soap/rpc/standaloneServer'  
require 'soap/wsdlDriver'
require 'soap/rpc/driver'
require 'utils.rb'
require 'rexml/document'
require 'dummy_node.rb'
require 'xsd/qname'
require 'soap/marshal'
require 'xsd/mapping'
require 'ControllerServiceDriver.rb'
require 'test/unit'
require 'soap/rpc/driver'
require 'webrick'
require 'webrick/httpproxy'
require 'logger'


class WSNService < SOAP::RPC::StandaloneServer  

  def initialize(* args)  
      super  
      
      @sensornodes = Hash.new 
      add_method(self, 'resetNodes', 'nodes')  
      add_method(self, 'areNodesAlive', 'nodes')
      add_method(self, 'send', 'nodeIds', 'message') 
      add_method(self, 'flashPrograms', 'stuff')  
      add_method(self, 'getNeighbourhood', 'node')
      add_method(self, 'getLinkQuality', 'src', 'dst')
      add_method(self, 'getPropertyValueOf', 'node', 'propertyName')
      add_method(self, 'getVersion')
      add_method(self, 'getNetwork')
      add_method(self, 'getControllerService')
      
      
      config = parse_config("/etc/TARWIS.conf");
          
         
      @requestNumber = 0;
      @flashThread = 0;
      @resetThread = 0;
      @areNodesAliveThread = 0;
      
      # t = Thread.new {
      #   while true
      #     # puts "#####NODECHECKERTHREAD#####"
      #     @sensornodes.keys.each { |urn|
      #         if @sensornodes[urn]
      #             t = @sensornodes[urn].getOutputThread()
      #             alive = t.alive?
      #             # puts "checking node urn: " + urn + "=" + alive.to_s   
      #         end
      #    }
      #     # puts "#####END#####"
      #     sleep(10);
      #   end
      # }
  end
  
  def getControllerService()
    controllerService = Controller.new();
    controllerService.options["protocol.http.protocol_version"] = "HTTP/1.0"
    controllerService.streamhandler.client.receive_timeout = 180
    return controllerService
  end
  
  def getRequestIdentifier(method)
    @requestNumber = @requestNumber + 1; 
    identifier = "req_" + method.to_s() + "#" + @requestNumber.to_s();
    return identifier
  end
     
  # hard-reset the nodes with the urn - e.g. by power-cycling or similia
  def resetNodes(*nodes)  
      puts "resetNodes called\n";
      var = nodes[0].to_s;    
      extracted = getNodesFromParams(var)
              
      identifier = getRequestIdentifier("resetNodes");
      if @resetThread!=0
        Thread.kill(@resetThread)
      end
      @resetThread = Thread.new {    
        resetNodesThreaded(extracted,identifier);
      }
      return identifier;
  end  
  

  
  def resetNodesThreaded(extracted, requestID)
     puts "resetNodesThreaded called reqID: " + requestID.to_s();
     sleep(3)    
     # print "nodes: ", extracted.length, "\n";
     
     # controllerService.wiredump_dev = STDERR
     extracted.length.times do |i|
       # print "trying to reset node ", extracted[i].to_s, "\n"
       if @sensornodes[extracted[i].to_s] 
          @sensornodes[extracted[i].to_s].resetNode()
       else
          print "resetNode called but node does not exist: ", extracted[i].to_s , "\n"
       end
       puts "updating resetNodes status of node " + extracted[i].to_s;  
       statusMsg = Status.new(extracted[i].to_s, 1, "resetNodes status update");
       requestStatus = RequestStatus.new(requestID, statusMsg);
       array = Array.new;
       array.push(requestStatus);
       receiveStatus = ReceiveStatus.new(array);
       getControllerService().receiveStatus(receiveStatus);
       puts "update call finished";  
    end            
    puts "resetNodesThreaded end\n";
  end
  
    
  def getNodesFromParams(var)
     # print "var: ", var  ,"\n"
     extracted = Array.new
     array = var.split(/urn:/)
     array.length.times do |i|
       node = array[i].to_s()
       node.gsub!(/[\n]+/, "\n");
       node.gsub!('\n','')
       if !node.empty?
         print "node: ", node  ,"\n"
         concated = "urn:" + node;
         extracted.push(concated)
       end
     end
     return extracted
  end
  
  
  def areNodesAlive(*nodes)  
     puts "areNodesAlive called\n";
     var = nodes[0].to_s;    
     extracted = getNodesFromParams(var)
          
     identifier = getRequestIdentifier("areNodesAlive");
     if @areNodesAliveThread!=0
         Thread.kill(@areNodesAliveThread);
     end
     @areNodesAliveThread = Thread.new {    
        areNodesAliveThreaded(extracted,identifier);
     }
     return identifier;
  end
      
  def areNodesAliveThreaded(extracted, requestID)
    puts "areNodesAliveThreaded called reqID: " + requestID.to_s();
    sleep(3)
         
    extracted.length.times do |i|
      # puts "updating status of node " + extracted[i].to_s;  
      randomTrueFalse = (rand(100) % 5 == 0);
      statusVar = randomTrueFalse ? 0:1;
      statusMsg = Status.new(extracted[i].to_s, statusVar, "areNodesAlive status update")
      requestStatus = RequestStatus.new(requestID, statusMsg)
      
      array = Array.new;
      array.push(requestStatus);
      receiveStatus = ReceiveStatus.new(array);
      getControllerService().receiveStatus(receiveStatus)


      # puts "callend receiveStatus"
    end            
    puts "areNodesAliveThreaded end\n";
  end
  

  
 
  def flashPrograms(*stuff)
      print "flashPrograms called at the (Dummy-)WSNService\n"
      extracted = getNodesFromParams(stuff[0].to_s)
      extracted.length.times do |i|
          print "flashing image to node " + extracted[i].to_s, "\n"
      end
      requestID = getRequestIdentifier("flashPrograms");
      
      @sensornodes.keys.each { |urn|
          puts "trying to kill node " + urn 
          @sensornodes[urn].kill
      }
      
      @sensornodes = Hash.new 
      @factory = DummyNodesFactory.new()
      
      if @flashThread!=0
        Thread.kill(@flashThread)
      end
      @flashThread = Thread.new {
          flashImagesThreaded(extracted, requestID);
      }
      puts "flashPrograms ended"
      return requestID;
  end  
  
  
  
  def flashImagesThreaded(extracted,requestID)
     puts "flashImagesThreaded called reqID: " + requestID.to_s();
     sleep(3)
     # controllerService.wiredump_dev = STDERR
         
     extracted.length.times do |i|
         # mimick sensor nodes by spawning some threads
         puts "i=" + i.to_s;
         puts "flashing node " +  extracted[i];
         dummyNode = @factory.spawnNewNode(extracted[i]);
         @sensornodes[extracted[i]]=dummyNode;
         statusProgress = 100;
         puts "updating status of node " + extracted[i].to_s + "=" + statusProgress.to_s;  
         # generate complexType SOAP object to pass to receiveStatus function
         statusMsg = Status.new(extracted[i].to_s, statusProgress, "flashPrograms status update")
         requestStatus = RequestStatus.new(requestID, statusMsg)
         
         array = Array.new;
         array.push(requestStatus);
         receiveStatus = ReceiveStatus.new(array);

         puts "reporting status " +  extracted[i];
         getControllerService().receiveStatus(receiveStatus)
         puts "status reported " +  extracted[i];
     end
     puts "flashImagesThreaded end";
  end
   
  #  if your sensor nodes support issuing commands (e.g. over the serial interface), 
  #  send the string "command" to the node urn 
  def send(nodeIds, message)  
     puts "WSNService: send called!";
     print "id: ", nodeIds.to_s, "\n";
     urn = nodeIds.to_s.strip()
     if @sensornodes[urn]
            @sensornodes[urn].runCommand("COMMAND")
     else
            puts "runCommand called (node object does not yet exist): " + urn;
     end

  end

  
  #  This WebService must return the adjacent neighbors of the node with the given urn
  #  How this is achieved is left open to the Implementer of the WebService
  #  E.g. in the UBERN implementation we search through the wisebed.sql tables 
  #  and check the sensornodes_output table for recently received packets
  
  def getNeighbourhood(node)  
       array = @sensornodes.keys
       if array.size > 0 
         id = getIDFromURN(node)
         # print "id: ", id , "\n"
         prefix = getPrefixFromURN(node)
         pseudoRandomNumber = (getMinutes.to_i + id.to_i) % array.size
         while array[pseudoRandomNumber].eql?(node) do
           pseudoRandomNumber = (getMinutes.to_i + id.to_i) % array.size
         end
         # print "pseudoRandomNumber: ", pseudoRandomNumber , "\n"
         # print "getNeighbourhood(", node, ") returning ", array[pseudoRandomNumber] , "\n"
         return array[pseudoRandomNumber]
       end    
       return
  end
  
  
  def get_file_as_string(filename)
    data = ''
    f = File.open(filename, "r") 
    f.each_line do |line|
        data += line
    end
    return data
  end

 
  #  This WebService must return a WiseML description of the sensor node 
  #  resources of the testbed. This is a tentative notion of this kind 
  #  of WiseML description, and may change while WiseML will develop
  def getNetwork() 
        
       config = parse_config("/etc/TARWIS.conf");
       puts "getNetwork called at the (RUBY) WSNService"
       filepath = config["TARWIS_DIRECTORY"].strip + "DUMMY_TESTBED_IMPLEMENTATION/DUMMY_TESTBED.wiseml";
       # puts "getNetwork: ", filepath
       
       graphDescription = get_file_as_string(filepath);
       
       return graphDescription;
  end
  
  def getVersion()  
      # print "getVersion called!\n";
      return "2.1";
  end  
  
  def getLinkQuality(src, dst)  
      return "1";
  end  
 
  def getPropertyValueOf(node, propertyName)  
      return "0.0";
  end  

 
end  
