package SessionManagementDriver;

use Env;
use Data::Dumper;
    
# read the configuration
my $conffileOK = readConfiguration("/etc/TARWIS.conf");
if(!$conffileOK) {
   print "configuration file not found or non-readable, exiting now!\n";
   exit(1);
}
    
    
sub getClient_CommonTypesURN {
    my $client = SOAP::Lite->new();
    $client->uri('urn:CommonTypes');
    $client->proxy($ENV{'SESSION_MANAGEMENT_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
    $client->soapversion('1.1');
    return $client;
}   
    
sub getClient_SessionManagerURN {
    my $client = SOAP::Lite->new();
    $client->uri('urn:SessionManagementService');
    $client->proxy($ENV{'SESSION_MANAGEMENT_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
    $client->soapversion('1.1');
    return $client;
}    

require "Utils.pm";

sub getSessionManagementDriver {
    my $client = SOAP::Lite->new();
    my $file = &Utils::correctPath($ENV{'TARWIS_DIRECTORY'}. "/WSDL/SessionManagementService.wsdl");
    &Utils::replaceEndpointURLinWSDL($file, $ENV{'SESSION_MANAGEMENT_SERVICE_URL'});
    $SessionManagementDriver = $client->service("file:" . $file);
    $client->proxy($ENV{'SESSION_MANAGEMENT_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
    return $SessionManagementDriver;
}

sub getNetwork {
	return  getClient_CommonTypesURN()->getNetwork()->result;
}

sub getConfiguration { 
    return  getClient_CommonTypesURN()->getConfiguration();
}

sub free {
    return  getClient_SessionManagerURN()->free(@_)->result;
}

sub areNodesAlive {
    return  getSessionManagementDriver()->areNodesAlive(@_);
}

our $raw_xml_soap_response;

sub setRawXML_SOAPResponse {
    $raw_xml_soap_response = $_[0];
}


sub getInstance {
	# print "SessionManagementDriver::getInstance\n";
    my $client = SOAP::Lite->new();
    my $file = &Utils::correctPath($ENV{'TARWIS_DIRECTORY'}. "/WSDL/SessionManagementService.wsdl");
    &Utils::replaceEndpointURLinWSDL($file, $ENV{'SESSION_MANAGEMENT_SERVICE_URL'});
    $SessionManagementDriver = $client->service("file:" . $file);
    $client->proxy($ENV{'SESSION_MANAGEMENT_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
    $SessionManagementDriver->deserializer(MyCustomDeserializer->new); 
          
    BEGIN {
       package MyCustomDeserializer; @MyCustomDeserializer::ISA = 'SOAP::Deserializer';
       use SOAP::Lite;
       use vars qw(@ISA);
       require "Utils.pm";
          
       sub deserialize {
          &SessionManagementDriver::setRawXML_SOAPResponse($_[1]);
          shift->SUPER::deserialize(@_);
       }
    }
    sleep(1);
    # print "calling getInstance\n";
    $SessionManagementDriver->getInstance(@_);
    # print "SOAP: $raw_xml_soap_response\n";
    my $method_name = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_response, "getInstanceResponse");
    # print "MethodName: $method_name\n";
    my $instance_url = &Utils::trimchomp(&Utils::getItemFromSOAPBody($raw_xml_soap_response, $method_name));
    if($instance_url eq "") {
    	$instance_url = &Utils::trimchomp(&Utils::getItemFromSOAPBody($raw_xml_soap_response, $method_name, "return"));
    }
    return $instance_url;
}



sub readConfiguration {
     my $conf = $_[0];
     open(CONFIG, $conf) or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}


    
1;
