package AsynchronousDeamons;

use SOAP::Lite; #  + 'trace'; 

use strict;
require "DatabaseInterface.pm";
require "WSNService.pm";
require "SessionManagementService.pm";
require "UBERNUtils.pm";
use Tie::ShareLite qw( :lock );
use IPC::ShareLite;
use Storable;
our $MAX_LINES = 2000;
use Time::HiRes qw( usleep ualarm gettimeofday tv_interval );

our $PINGER_SLEEP_INTERVAL             = 10;
our $ALIVESTATUS_DEAMON_SLEEP_INTERVAL = 5;
our $RESET_DEAMON_SLEEP_INTERVAL       = 2;
our $FLASH_DEAMON_SLEEP_INTERVAL       = 2;
our $HARM_INTERVAL                     = 300;

sub MeshnodePinger {
    my $outfile = $_[0]; 
    my $errfile = $_[1]; 
    
    # actually, these are not host names (anymore) that are resolved by dns
    # but simple entries in /etc/hosts in gridlab23
    
    my @meshnodes = (  "wb00.unibe.ch",
                       "wb01.unibe.ch",
                       "wb02.unibe.ch",
                       "wb03.unibe.ch",
                       "wb04.unibe.ch",
                       "wb05.unibe.ch",
                       "wb06.unibe.ch",
                       "wb07.unibe.ch",
                       "wb08.unibe.ch",
                       "wb09.unibe.ch",
                       "wb10.unibe.ch",
                       "wb11.unibe.ch",
                       "wb12.unibe.ch",
                       "wb13.unibe.ch",
                       "wb14.unibe.ch",
                       "wb15.unibe.ch",
                       "wb16.unibe.ch",
                       "wb17.unibe.ch",
                       "wb18.unibe.ch",
                       "wb19.unibe.ch",
                       "wb20.unibe.ch",
                       "wb21.unibe.ch",
                       "wb22.unibe.ch",
                       "wb23.unibe.ch",
                       "wb24.unibe.ch",
                       "wb25.unibe.ch",
                       "wb26.unibe.ch",
                       "wb27.unibe.ch",
                       "wb28.unibe.ch",
                       "wb29.unibe.ch",
                       "wb30.unibe.ch",
                       "wb31.unibe.ch",
                       "wb32.unibe.ch",
                       "wb33.unibe.ch",
                       "wb34.unibe.ch",
                       "wb35.unibe.ch",
                       "wb36.unibe.ch",
                       "wb37.unibe.ch",
                       "wb38.unibe.ch",
                       "wb39.unibe.ch"
    );
    
    my @sensornodes_msb    = ("ee00.unibe.ch", 
                              "ee01.unibe.ch",
                              "ee02.unibe.ch",
                              "ee03.unibe.ch",
                              "ee04.unibe.ch", 
                              "ee05.unibe.ch", 
                              "ee06.unibe.ch");                
    
    
    &UBERNUtils::redirectOutput($outfile,$errfile);
    
    $0 = "MeshnodePinger";
    
    
    while(1) {
    	my $database = new DatabaseInterface();
    	my $sky_alive = 0;
        my $sky_dead  = 0;
        my $msb_alive = 0;
        my $msb_dead  = 0;
        
    	foreach my $node (@meshnodes){
    		my $sensor = &UBERNUtils::getSensorNodeOfMeshNode($node);
    		$sensor = &UBERNUtils::generateURN($sensor);
    		# print "iterating to node: $node - sensor: $sensor " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
    		if(!$database->isInAsynchronousStateSecs($sensor, $HARM_INTERVAL)) {
	            my $state = &UBERNUtils::pingMeshnode($node);
	            $database->setAliveStateOfNode($sensor, $state);
	            if($state>0) {
	            	$sky_alive += 1;
	            }
	            else {
	            	$sky_dead += 1;
	            }
    		}
    		else {
    			$sky_alive += 1;
    			$database->setAliveStateOfNode($sensor, 1);
    			print "node $node is currently being flashed or reset " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
    		}
    	}
    	foreach my $node (@sensornodes_msb){
            my $sensor = &UBERNUtils::getSensorNodeOfEEPC($node);
            $sensor = &UBERNUtils::generateURN($sensor);
            # print "iterating to node: $node - sensor: $sensor " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
    		if(!$database->isInAsynchronousStateSecs($sensor, $HARM_INTERVAL)) {
	            &UBERNUtils::pingMeshnode($node);
	            my $state = &UBERNUtils::pingMeshnode($node);
	            $database->setAliveStateOfNode($sensor, $state);
	            if($state>0) {
                    $msb_alive += 1;
                }
                else {
                    $msb_dead += 1;
                }
    		}
    		else {
    			$msb_alive += 1;
    			$database->setAliveStateOfNode($sensor, 1);
                print "node $node is currently being flashed or reset " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
            }
        }
        print "done with loop through all nodes: TMotes alive/dead: $sky_alive/$sky_dead MSB alive/dead: $msb_alive/$msb_dead " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";        
        sleep($PINGER_SLEEP_INTERVAL);
        &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    }
}


sub IsAliveStatusServerDeamon {
	my $outfile = $_[0]; 
	my $errfile = $_[1]; 
	
	
	&UBERNUtils::redirectOutput($outfile,$errfile);
	
	
	
	$0 = "IsAliveStatusServerDeamon";
 	
	while(1) {
		sleep($ALIVESTATUS_DEAMON_SLEEP_INTERVAL);
		my $database = new DatabaseInterface();
	    $database->owner("IsAliveStatusServerDeamon");
	    my $requestID = $database->getNextUnfinishedRequestIDOfType("areNodesAlive");
	    if($requestID != -1) {
			print "found job! executing requestID=$requestID ". &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
		    sleep(5);
			my @nodes = $database->getNodesOfRequest($requestID);
			&SessionManagementService::areNodesAliveOperations(\@nodes, $requestID);
	    	$database->setAsynchronousRequestDoneStatus($requestID, "areNodesAlive", 1);
	    	print "job $requestID completed ... continuing\n";	
	    	&UBERNUtils::truncateFile($outfile, $MAX_LINES);
	    	print "done with requestID=$requestID " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
		}
	}
}

sub ResetOperationDeamon {
	my $outfile = $_[0]; 
	my $errfile = $_[1]; 
	
	
	&UBERNUtils::redirectOutput($outfile,$errfile);
	   
	
	$0 = "ResetOperationDeamon";
 	
	while(1) {
		sleep($RESET_DEAMON_SLEEP_INTERVAL);
		my $database = new DatabaseInterface();
        $database->owner("ResetOperationDeamon");
		my $requestID = $database->getNextUnfinishedRequestIDOfType("resetNodes");
		if($requestID != -1) {
			print "ResetOperationDeamon: request found executing requestID=$requestID!" . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
			my @nodes = $database->getNodesOfRequest($requestID);
			&WSNService::doResetOperations(\@nodes, $requestID);
	    	
	    	$database->setAsynchronousRequestDoneStatus($requestID, "resetNodes", 1);
	    	print "job $requestID completed ... continuing\n";	
	    	&UBERNUtils::truncateFile($outfile, $MAX_LINES);
	    	print "done with requestID=$requestID " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
		}
	}
}

sub FlashOperationDeamon {	
	
	my $outfile = $_[0]; 
	my $errfile = $_[1]; 
	
	
	&UBERNUtils::redirectOutput($outfile,$errfile);
	
	use POSIX qw(:signal_h :errno_h :sys_wait_h);
	
	$SIG{CHLD} = \&REAPER;
    sub REAPER {
        my $pid;
        $pid = waitpid(-1, &WNOHANG);
        if ($pid == -1) {
        } elsif (WIFEXITED($?)) {
            print "Process $pid exited.\n";
        } else {
            print "False alarm on $pid.\n";
        }
        $SIG{CHLD} = \&REAPER;          
    }
   
	
	$0 = "FlashOperationDeamon";		
 	
	while(1) {
		sleep($FLASH_DEAMON_SLEEP_INTERVAL);
	    my $database = new DatabaseInterface();
	    $database->owner("FlashOperationDeamon");
		my $requestID = $database->getNextUnfinishedRequestIDOfType("flashPrograms");
		if($requestID != -1) {
			my $pid = fork();
            if( $pid == 0 ){
            	sleep(3);
            	print "found job! executing requestID=$requestID " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
                my $databaseObject = new DatabaseInterface();
            	$databaseObject->setAsynchronousRequestDoneStatus($requestID, "flashPrograms", 1);
                &WSNService::doFlashOperations($requestID);
	    		print "job $requestID completed ... continuing\n";
		    	print "done with requestID=$requestID " . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
	            exit(0);            
	        }
	        else {
                print "deployed child process with asynchronous request: $requestID\n";
            }	        
		    &UBERNUtils::truncateFile($outfile, $MAX_LINES);    
		}
	}
}


1;
