#!/usr/bin/perl

use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use HTTP::Request;
use HTTP::Response;
use LWP::UserAgent;
use File::Slurp;
use Env;

$SIG{INT}  = \&exitProcess;
$SIG{KILL} = \&exitProcess;
$SIG{TERM} = \&exitProcess;



readConfiguration("/etc/TARWIS.conf");
push(@INC, $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION");
require "UBERNUtils.pm";
our $CURRENT_DIRECTORY              = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION");  
our $MSB_COMMAND_PORT               = 4000;    

sub getMeshNodeOfSensorNode {
    my $sensorNode_id = $_[0];
    my $mesh_id = $sensorNode_id-1;
    if($mesh_id<10) {
       $mesh_id = "0".$mesh_id;
    }
    return "wb$mesh_id";
}


sub isInteger {
    if ( $_[0] =~ /^\d+$/ ) { 
       return 1;
    } 
    return 0;
}


my $nodeid1 = $ARGV[0];
my $file1   = $ARGV[1];

my $ua = LWP::UserAgent->new;
$ua->timeout(200);

$0 = "DeployImage_$nodeid1";

our $meshnode_hostname = getMeshNodeOfSensorNode($nodeid1);

        
if(isInteger($nodeid1)) {
   read_file( $file1 , binmode => ':raw' , scalar_ref => 1 );
}
    
my $url = "http://$meshnode_hostname.unibe.ch/control/flashNodes.php";
my $response = $ua->post( $url,
                        Content_Type => 'form-data',
                        Content => [ "file1" => ["$file1"], "nodeid1" => $nodeid1 ],
                        );   

if( $response->content =~ /succeeded/ ) {
  exit(0)
}
else {
  exit(1);
}

sub readConfiguration {
     my $conf = $_[0];
     if(!-e $conf || !-r $conf) {
        print "Configuration file: $conf does not exist!\n";
        return 0;
     }
     open(CONFIG, $conf)  or return 0;   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

sub exitProcess {
     print "DeployImage.pl dedicated to $meshnode_hostname: signal caught! exiting " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
     exit(1);  
}