#!/usr/bin/perl

use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use IPC::ShareLite;
use Env;
use Pod::WSDL;

readConfiguration("/etc/TARWIS.conf");

push(@INC, $ENV{"TARWIS_DIRECTORY"});
push(@INC, $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION");

our $ACTIVATE_TELOSB_NODES = 1;
our $ACTIVATE_MSB430_NODES = 1;

require "SensorNodesOutputHandler.pm";
require "UBERN_SOAPServer.pm";
require "AsynchronousDeamons.pm";
require "SessionManagementService.pm";
require "WSNService.pm";
require "DatabaseInterface.pm";
   
print "Starting UBERN TestbedAccess Implementation...\n\n\n";


print "starting SessionManagementService...\n";
print "starting IsAliveStatusServerDeamon...\n";
print "starting ResetOperationDeamon...\n";
print "starting FlashOperationDeamon...\n";
print "starting SensorNodesOutputHandlers...\n\n";
print "TestbedAccessMainBERN done\n\n";

my $database = new DatabaseInterface();    
$database->resetAsynchronousOperationTable();

&UBERNUtils::daemonize(\&UBERN_SOAPServer::SessionManagementServiceSOAPServer, "SessionManagementServiceSOAPServer");

&UBERNUtils::daemonize(\&AsynchronousDeamons::IsAliveStatusServerDeamon, "IsAliveStatusServerDeamon");

&UBERNUtils::daemonize(\&AsynchronousDeamons::ResetOperationDeamon, "ResetOperationDeamon");

&UBERNUtils::daemonize(\&AsynchronousDeamons::FlashOperationDeamon, "FlashOperationDeamon");

&UBERNUtils::daemonize(\&AsynchronousDeamons::MeshnodePinger, "MeshnodePinger");


if($ACTIVATE_TELOSB_NODES) {
    &SensorNodesOutputHandler::startSensorNodesOutputHandlerDaemons_TelosB();
}

if($ACTIVATE_MSB430_NODES) {
    &SensorNodesOutputHandler::startSensorNodesOutputHandlerDaemons_MSB430();
}

&SensorNodesOutputHandler::startOutputProcessor();

&SensorNodesOutputHandler::startGridlabMaintenanceReboot();

# MOCK SERVICE FOR THE WISEBED.EU webpage

startMockService();

sub readConfiguration {
     my $conf = $_[0];
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

sub startMockService {
    my $rubySessionManager = "ruby " . $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION/" . "main.rb"; 
    system($rubySessionManager);
}