use SOAP::Lite + 'trace'; 

use strict;
use IPC::ShareLite;
use HTTP::Request;
use HTTP::Response;
use Env;
use Pod::WSDL;
use Data::Dumper;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use Time::HiRes qw(gettimeofday);

readConfiguration("/etc/TARWIS.conf");

# deliverStatusSOAP("urn:ubern:1", 1, "request#1", "status update");
# deliverMessageSOAP("urn:ubern:1", "blah blah blah");

isAlive();

sub getControllerServiceDriver {
   my $driver = SOAP::Lite->service("file:/opt/TARWISServer/WSDL/ControllerService.wsdl");
   $driver->uri('urn:ControllerService');
   return $driver;
}

sub isAlive {
   getControllerServiceDriver()->isAlive();
}

sub deliverStatusSOAP {
   my $urn    = $_[0];
   my $status = $_[1];
   my $requestID = $_[2]; 
   my $msg = $_[3];
   my $soap_message = generateComplexStatusMessage($requestID, $urn, $status, $msg);
   getControllerServiceDriver()->receiveStatus($soap_message);
}

sub deliverMessageSOAP {
   my $urn    = $_[0];
   my $msg    = $_[1];
   my $soap_message = generateComplexTypeMessageText($urn, $msg);
   getControllerServiceDriver()->receive($soap_message);
}

sub generateComplexStatusMessage {
  
   my $requestId     = $_[0];
   my $nodeId        = $_[1];
   my $value         = $_[2];
   my $msg           = $_[3];
 
   my $soap_message = SOAP::Data->name("status" => \SOAP::Data->value( 
                                                  SOAP::Data->new(name => "requestId",  value => $requestId), 
                                                  SOAP::Data->name("status" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                                SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                                SOAP::Data->new(name => "msg", value => $msg , type => "string")
                                                                                                  )
                                                                  ) )
                                       ); 
   return $soap_message;                                    
}

sub getTime() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   my $time = sprintf("%4d-%02d-%02dT%02d:%02d:%02dZ", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}


sub generateComplexTypeMessageText {
  
   my $node_urn     = $_[0];
   my $textData     = $_[1];

   
   my $binaryData   = "";
   my $binaryType   = "";
   
   my $soap_message = SOAP::Data->name("msg" => \SOAP::Data->value( 
                                                     SOAP::Data->new(name => "sourceNodeId",  value => $node_urn, type => "string"), 
                                                     SOAP::Data->new(name => "timestamp",     value => getTime(), type => "dateTime"), 
                                                     SOAP::Data->name("textMessage" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "msg", value => $textData, type => "string"),
                                                                                                  )
                                                                  ),
                                                                       )
                                       ); 
   return $soap_message;                                    
}


sub readConfiguration {
     my $conf = $_[0];
     if(!-e $conf || !-r $conf) {
        print "Configuration file: $conf does not exist!\n";
        return 0;
     }
     open(CONFIG, $conf)  or return 0;   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}
