
CREATE DATABASE wsn_info;
USE wsn_info;


CREATE TABLE aliveState (
    node_urn VARCHAR(50),
    node_state INT,
    PRIMARY KEY(node_urn)
);

CREATE TABLE asynchronousRequestTypeAndDoneStatus (
    request_id   VARCHAR(50), 
    request_type VARCHAR(50), 
    done BOOLEAN,
    PRIMARY KEY(request_id)
);

CREATE TABLE outputBuffer (
    node_urn VARCHAR(50),
    line VARCHAR(500),
    time DATETIME,
    usecs INT,
    rowindex INT PRIMARY KEY AUTO_INCREMENT,
    UNIQUE(node_urn, line, time, usecs)
);


CREATE TABLE asynchronousOperation (
    request_id VARCHAR(50), 
    node_urn VARCHAR(50),
    node_state INT,
    time DATETIME,
    PRIMARY KEY(request_id, node_urn)
);

CREATE TABLE properties(
    prop_id INT PRIMARY KEY AUTO_INCREMENT,
    description VARCHAR(50),
    type VARCHAR(50),
    unit VARCHAR(50),
    defaultvalue VARCHAR(50),
    CONSTRAINT uniqueDescription UNIQUE (description)
);

CREATE TABLE setup (
    origin_x DOUBLE PRECISION,
    origin_y DOUBLE PRECISION,
    origin_z DOUBLE PRECISION,
    phi DOUBLE PRECISION,
    theta DOUBLE PRECISION,
    coordinateType VARCHAR(50),
    description VARCHAR(200)
);

CREATE TABLE links_setup (
    source        VARCHAR(50) NOT NULL,
    target        VARCHAR(50) NOT NULL,
    datatype      VARCHAR(50) NOT NULL,
    unit          VARCHAR(50) NOT NULL,
    defaultvalue  VARCHAR(50) NOT NULL,
    encrypted     VARCHAR(50) NOT NULL,
    virtual       VARCHAR(50) NOT NULL,
    PRIMARY KEY(source,target)
);



CREATE TABLE sensornodes_positions(
    node_urn VARCHAR(50) PRIMARY KEY,
    x DOUBLE PRECISION,
    y DOUBLE PRECISION,
    z DOUBLE PRECISION,
    original_x DOUBLE PRECISION,
    original_y DOUBLE PRECISION,
    original_z DOUBLE PRECISION
);

CREATE TABLE sensornodes (
	node_urn VARCHAR(50),
	prop_id INT,
	value VARCHAR(50),
	time DATETIME,
	usecs INT,	
	PRIMARY KEY(node_urn, prop_id)
);

CREATE TABLE sensornodes_output(
	node_urn VARCHAR(50),
	exp_id VARCHAR(100) NOT NULL, 
	line VARCHAR(500),
	time DATETIME,
	usecs INT,
	rowindex INT PRIMARY KEY AUTO_INCREMENT,
	UNIQUE(node_urn, line, time, usecs)
);

CREATE TABLE sensorvalues (
    node_urn VARCHAR(50),
    prop_id INT,
    value VARCHAR(50),
    time DATETIME,
    usecs INT, 
    PRIMARY KEY(node_urn, prop_id, value, time, usecs)
);


CREATE TABLE sensornode_network (
    node_urn VARCHAR(50),
    node_urn_neighbor VARCHAR(50),
    ETX INT,
    time DATETIME,
    usecs INT,
    PRIMARY KEY(node_urn, node_urn_neighbor)
);


CREATE TABLE experimentcontroller_output(
    exp_id VARCHAR(100) NOT NULL,
    line VARCHAR(500),
    time DATETIME,
    usecs INT,
    rowindex INT PRIMARY KEY AUTO_INCREMENT
);


CREATE TABLE images (
	image_id INTEGER PRIMARY KEY AUTO_INCREMENT,
	image_name VARCHAR(100) NOT NULL,
	image_version TEXT,
	image_platform TEXT,
	image_description TEXT,
	image_file_name TEXT,
    image_file_size INT,
	image_file LONGBLOB,
	image_time DATETIME,
	image_usecs INT,
	image_visible BOOLEAN,
	exp_user_unique_id VARCHAR(100) NOT NULL ,
	CONSTRAINT uniqueName UNIQUE (image_name, exp_user_unique_id)
);


CREATE TABLE users (
    user_unique_id VARCHAR(100) NOT NULL PRIMARY KEY,
    user_firstname  VARCHAR(100) NOT NULL,
    user_name  VARCHAR(100) NOT NULL,
    user_email VARCHAR(100) NOT NULL    
);


CREATE TABLE experiment_network ( 
    exp_id VARCHAR(100) NOT NULL, 
    node_urn VARCHAR(50), 
    image_id INT, 
    PRIMARY KEY(exp_id, node_urn) 
);

CREATE TABLE running_experiments (
    exp_id VARCHAR(100), 
    wsn_instance_url VARCHAR(100), 
    exp_start_time DATETIME, 
    exp_end_time DATETIME,
    PRIMARY KEY(exp_id) 
);

CREATE TABLE experiment_info (
    exp_id VARCHAR(100) NOT NULL PRIMARY KEY, 
    exp_name TEXT, 
    exp_description TEXT, 
    exp_start_time DATETIME, 
    exp_end_time DATETIME, 
    exp_user_unique_id VARCHAR(100), 
    exp_user_email VARCHAR(100),
    exp_result LONGBLOB,
    exp_visibility BOOLEAN,
    exp_done BOOLEAN,
    exp_runs INT
);

CREATE TABLE runtimeCommands (
    node_urn VARCHAR(50),
    command VARCHAR(50), 
    time DATETIME,
    usecs INT,
    experiment_id VARCHAR(100) NOT NULL, 
    done BOOLEAN,
    PRIMARY KEY(experiment_id, node_urn, command, time, usecs)
);

CREATE TABLE automatedCommands (
    command_id INT PRIMARY KEY AUTO_INCREMENT,
    exp_id VARCHAR(100),
    node_urn VARCHAR(50), 
    command VARCHAR(50),
    seconds_after_flash INT,
    UNIQUE (exp_id, node_urn, command, seconds_after_flash)
);

CREATE TABLE templated_commands (
    template_id VARCHAR(100),
    node_urn VARCHAR(50), 
    command VARCHAR(50),
    seconds_after_flash INT,
    PRIMARY KEY (template_id, node_urn, command, seconds_after_flash)
);

CREATE TABLE templated_nodes_images (
    template_id VARCHAR(100),
    node_urn VARCHAR(50), 
    image_id INTEGER,
    PRIMARY KEY (template_id, node_urn, image_id)
);

CREATE TABLE experiment_templates (
    template_uniqueID VARCHAR(100) NOT NULL PRIMARY KEY, 
    template_name VARCHAR(100), 
    template_description TEXT, 
    template_user_unique_id VARCHAR(100), 
    template_visibility BOOLEAN,
    template_runs INT
);
