<?php

require_once 'System.php';
require_once 'GraphViz.php';


class Image_GraphVizNG extends Image_GraphViz
{
    function Image_GraphVizNG($directed = TRUE, $attributes = array(), $name = NULL)
    {
        parent::__construct($directed, $attributes, $name);
    }
    
    function containsEdge($edge) {
            if (is_array($edge)) {
              $from = key($edge);
              $to   = $edge[$from];
              $id1   = $from . '_' . $to;
                            $id2   = $to . '_' . $from;

            if (isset($this->graph['edges'][$id1]) || isset($this->graph['edges'][$id2])) {
              return true;
            } else {
                return false;
            }
        }
    }

     /**
     * Parse the graph into GraphViz markup.
     *
     * @return string  GraphViz markup
     * @access public
     */
    function parse()
    {
        //Change: digraph -> graph
        if (isset($this->graph['name']) && is_string($this->graph['name'])) {
        if($this->graph['directed']) { 
            $parsedGraph = "digraph " . $this->graph['name'] . " {\n";
        } else {
            $parsedGraph = "graph " . $this->graph['name'] . " {\n";
        }
        } else {
        if($this->graph['directed']) { 
            $parsedGraph = "digraph G {\n";
        } else {
            $parsedGraph = "graph G {\n";
        }
        }

        if (isset($this->graph['attributes'])) {
            foreach ($this->graph['attributes'] as $key => $value) {
                $attributeList[] = $key . '="' . $value . '"';
            }

            if (!empty($attributeList)) {
                $parsedGraph .= 'graph [ '.implode(',', $attributeList) . " ];\n";
            }
        }

        if (isset($this->graph['nodes'])) {
            foreach($this->graph['nodes'] as $group => $nodes) {
                if ($group != 'default') {
                    $parsedGraph .= sprintf(
                      "subgraph \"cluster_%s\" {\nlabel=\"%s\";\n",

                      $group,
                      isset($this->graph['clusters'][$group]) ? $this->graph['clusters'][$group]['title'] : ''
                    );

                    if (isset($this->graph['clusters'][$group]['attributes'])) {
                        unset($attributeList);

                        foreach ($this->graph['clusters'][$group]['attributes'] as $key => $value) {
                            $attributeList[] = $key . '="' . $value . '"';
                        }

                        if (!empty($attributeList)) {
                            $parsedGraph .= implode(',', $attributeList) . ";\n";
                        }
                    }
                }

                foreach($nodes as $node => $attributes) {
                    unset($attributeList);

                    foreach($attributes as $key => $value) {
                            //Change: script tests, if value is a HTML Label (http://graphviz.org/doc/info/shapes.html#html). if yes, no "" at the beginning and end of the value ar written to .dot file
                            $firstValueChar = substr($value, 1, 1);
                            $lastValueChar = substr($value, -1, 1);
                            if($firstValueChar == '<' && $lastValueChar == '>') {
                                $attributeList[] = $key . '=' . $value . '';
                            } else {
                                $attributeList[] = $key . '="' . $value . '"';
                            }
                    }

                    if (!empty($attributeList)) {
                        $parsedGraph .= sprintf(
                          "\"%s\" [ %s ];\n",
                          addslashes(stripslashes($node)),
                          implode(',', $attributeList)
                        );
                    }
                }

                if ($group != 'default') {
                  $parsedGraph .= "}\n";
                }
            }
        }

        if (isset($this->graph['edges'])) {
            foreach($this->graph['edges'] as $label => $node) {
                unset($attributeList);

                $from = key($node);
                $to   = $node[$from];

                foreach($this->graph['edgeAttributes'][$label] as $key => $value) {
                    $attributeList[] = $key . '="' . $value . '"';
                }
                                
                                //change: prints now -- if graph is not directed
                $parsedGraph .= sprintf(
                  '"%s" %s "%s"',
                  addslashes(stripslashes($from)),
                  $this->graph['directed'] ? "->" : "--",
                  addslashes(stripslashes($to))
                );
                
                if (!empty($attributeList)) {
                    $parsedGraph .= sprintf(
                      ' [ %s ]',
                      implode(',', $attributeList)
                    );
                }

                $parsedGraph .= ";\n";
            }
        }

        return $parsedGraph . "}\n";
    }
}
?>
