DROP TABLE IF EXISTS User;

CREATE TABLE User (
    user_id INT NOT NULL AUTO_INCREMENT,
    user_privilege INT NOT NULL,
    user_uid VARCHAR(255) NOT NULL,
    user_last_login TIMESTAMP NULL,
    user_valid_until TIMESTAMP NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (user_id),
    INDEX user_uid_idx (user_uid)
);


# --------------------------------------------------------
#
# Table structure for table 'UserPrivilege'
#

DROP TABLE IF EXISTS UserPrivilege;

CREATE TABLE UserPrivilege (
    userprivilege_id INT NOT NULL AUTO_INCREMENT,
    userprivilege_name VARCHAR(128) NOT NULL,
    userprivilege_description VARCHAR(255) NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (userprivilege_id)
);


# --------------------------------------------------------
#
# Table structure for table 'Attribute'
#

DROP TABLE IF EXISTS Attribute;

CREATE TABLE Attribute (
    attribute_id INT NOT NULL AUTO_INCREMENT,
    attribute_trusted TINYINT DEFAULT '0' NOT NULL,
    attribute_type INT NOT NULL,
    attribute_user INT NOT NULL,
    attribute_value VARCHAR(255) NOT NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (attribute_id)
);

# --------------------------------------------------------
#
# Table structure for table 'UserAttributes'
#

DROP TABLE IF EXISTS UserAttributes;

CREATE TABLE UserAttributes (
    user_id INT NOT NULL,
    user_uid VARCHAR(255) NOT NULL,
    surname VARCHAR(255),
    givenName VARCHAR(255),
    commonName VARCHAR(255),
    privilegeName VARCHAR(255),
    swissEduPersonBirthdate VARCHAR(255),
    swissEduPersonGender VARCHAR(255),
    preferredLanguage VARCHAR(255),
    mail VARCHAR(255),
    homePostalAddress VARCHAR(255),
    postalAddress  VARCHAR(255),
    homePhone  VARCHAR(255),
    telephoneNumber  VARCHAR(255),
    mobileTelephoneNumber  VARCHAR(255),
    swissEduPersonHomeOrganization  VARCHAR(255),
    swissEduPersonHomeOrganizationType  VARCHAR(255),
    eduPersonAffiliation  VARCHAR(255),
    swissEduPersonStudyBranch1  VARCHAR(255),
    swissEduPersonStudyBranch2   VARCHAR(255),
    swissEduPersonStudyBranch3   VARCHAR(255),
    swissEduPersonStudyLevel  VARCHAR(255),
    swissEduPersonStaffCategory  VARCHAR(255),
    swissEduPersonOrgDN  VARCHAR(255),
    swissEduPersonOrgUnitDN  VARCHAR(255),
    swissEduPersonEntitlement  VARCHAR(255),
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (user_id),
    INDEX user_uid_idx (user_uid),
    INDEX commonName_idx (commonName),
    INDEX privilegename_idx (privilegeName),
    INDEX surname_idx (surname),
    INDEX givenname_idx (givenName)
);


# --------------------------------------------------------
#
# Table structure for table 'AttributeType'
#

DROP TABLE IF EXISTS AttributeType;

CREATE TABLE AttributeType (
    attributetype_id INT NOT NULL AUTO_INCREMENT,
    attributetype_standard TINYINT DEFAULT '0' NOT NULL,
    attributetype_name VARCHAR(128) NOT NULL,
    attributetype_ldapname VARCHAR(128) NULL,
    attributetype_description VARCHAR(255) NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (attributetype_id),
    INDEX attributetype_ldapname_idx (attributetype_ldapname)
);


# --------------------------------------------------------
#
# Table structure for table 'Policy'
#

DROP TABLE IF EXISTS Policy;

CREATE TABLE Policy (
    policy_id INT NOT NULL AUTO_INCREMENT,
    policy_attributetype INT NOT NULL,
    policy_attributevalue VARCHAR(255) NULL,
    policy_role INT NOT NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (policy_id)
);


# --------------------------------------------------------
#
# Table structure for table 'Role'
#

DROP TABLE IF EXISTS Role;

CREATE TABLE Role (
    role_id INT NOT NULL AUTO_INCREMENT,
    role_key VARCHAR(40) NOT NULL,
    role_is_visible TINYINT DEFAULT '0' NOT NULL,
    # 0: closed
    # 1: suspended
    # 2: open 
    role_state INT DEFAULT '0'  NOT NULL,
    role_user INT NOT NULL,
    role_title VARCHAR (255) NOT NULL,
    role_url VARCHAR (255) NOT NULL,
    role_adapter VARCHAR (255) NOT NULL,
    role_description TEXT NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (role_id)
);

# --------------------------------------------------------
#
# Table structure for table 'RoleManager'
#

DROP TABLE IF EXISTS RoleManager;

CREATE TABLE RoleManager (
    user_id INT NOT NULL,
    role_id INT NOT NULL,
    PRIMARY KEY (user_id,role_id)
);

# --------------------------------------------------------
#
# Table structure for table 'Subscription'
#

DROP TABLE IF EXISTS Subscription;

CREATE TABLE Subscription (
    # 0: pending 
    # 1: accepted
    # 2: suspended
    # 3: rejected
    subscription_state INT DEFAULT '0' NOT NULL,
    subscription_role_visited TINYINT DEFAULT '0' NOT NULL,
    subscription_user INT NOT NULL,
    subscription_role INT NOT NULL,
    subscription_counter INT DEFAULT '0' NOT NULL,
    subscription_last_login TIMESTAMP NULL,
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    date_modified TIMESTAMP NULL,
    addr_modified VARCHAR(32) NULL,
    PRIMARY KEY (subscription_user, subscription_role)
);

# --------------------------------------------------------
#
# Table structure for table 'Invitation'
#

DROP TABLE IF EXISTS Invitation;

CREATE TABLE Invitation (
    role_id INT NOT NULL,
    email VARCHAR(100) NOT NULL,
    token VARCHAR(100) NOT NULL,
    user_id_created INT DEFAULT '-1',
    date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    PRIMARY KEY  (role_id,email)
);

# -----------------------------------------------------------
#
# Table structure for table 'RoleAdapterParam'
#

DROP TABLE IF EXISTS RoleAdapterParam;

CREATE TABLE RoleAdapterParam (
  role_id INT NOT NULL,
  name VARCHAR(255) DEFAULT '' NOT NULL,
  value VARCHAR(255) DEFAULT '' NOT NULL,
  PRIMARY KEY (role_id, name)
);

# -----------------------------------------------------------
#
# Table structure for table 'Event'
#
DROP TABLE IF EXISTS Event;

CREATE TABLE Event (
    event_id INT NOT NULL AUTO_INCREMENT,
    event_type INT NOT NULL,
    event_level INT NOT NULL,
    event_message VARCHAR(255) NOT NULL,
    event_user INT NOT NULL,
    event_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    addr_created VARCHAR(32) DEFAULT '127.0.0.1' NOT NULL,
    PRIMARY KEY (event_id),
    INDEX event_date_idx (event_date),
    INDEX event_type_idx (event_type),
    INDEX event_level_idx (event_level)
);

# --------------------------------------------------------
#
# Table structure for table 'Accounting'
#
DROP TABLE IF EXISTS Accounting;



# MySQL Attribute Queries

# --------------------------------------------------------------------
# insert standard attribute types 
# --------------------------------------------------------------------
delete from AttributeType where attributetype_id < 23;

insert into AttributeType values (1, 1, 'Family Name', 'surname', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (2, 1, 'Given Name', 'givenName', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (3, 1, 'Birthdate', 'swissEduPersonBirthdate', '',  NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (4, 1, 'Gender', 'swissEduPersonGender', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (5, 1, 'Language', 'preferredLanguage', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (6, 1, 'Email', 'mail', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (7, 1, 'Home Address', 'homePostalAddress', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (8, 1, 'Postal Address', 'postalAddress','',  NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (9, 1, 'Private Phone', 'homePhone', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (10, 1, 'Business Phone', 'telephoneNumber', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (11, 1, 'Mobile Phone', 'mobileTelephoneNumber', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (12, 1, 'Home Organization', 'swissEduPersonHomeOrganization', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (13, 1, 'Home Organization Type', 'swissEduPersonHomeOrganizationType', '',  NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (14, 1, 'Home Organization Affiliation', 'eduPersonAffiliation', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (15, 1, 'Study Branch 1', 'swissEduPersonStudyBranch1',  '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (16, 1, 'Study Branch 2', 'swissEduPersonStudyBranch2', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (17, 1, 'Study Branch 3', 'swissEduPersonStudyBranch3', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (18, 1, 'Study Level (Branch 3)', 'swissEduPersonStudyLevel', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (19, 1, 'Staff Category', 'swissEduPersonStaffCategory', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (20, 1, 'Organization Path', 'swissEduPersonOrgDN', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (21, 1, 'Organization Unit Path', 'swissEduPersonOrgUnitDN', '',  NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (22, 1, 'Group Membership', 'swissEduPersonEntitlement', '', NULL, '127.0.0.1', NULL, NULL);
insert into AttributeType values (23, 1, 'Local Password', 'localPassword', '', NULL, '127.0.0.1', NULL, NULL);

# --------------------------------------------------------------------
# inserting user privileges
# --------------------------------------------------------------------

INSERT INTO UserPrivilege VALUES (1, 'default', 'User', NULL, '127.0.0.1', NULL, NULL);
INSERT INTO UserPrivilege VALUES (2, 'role_admin', 'Role Administrator', NULL, '127.0.0.1', NULL, NULL);
INSERT INTO UserPrivilege VALUES (3, 'portal_admin', 'Portal Administrator', NULL, '127.0.0.1', NULL, NULL);
INSERT INTO UserPrivilege VALUES (4, 'subscription_admin', 'Subscription Administrator', NULL, '127.0.0.1', NULL, NULL);

# --------------------------------------------------------------------
# inserting predefined users  
# --------------------------------------------------------------------

INSERT INTO User VALUES (1, 3, 'portaladmin', NULL, NULL, NULL, '127.0.0.1', NULL, NULL);
INSERT INTO UserAttributes (user_id,user_uid) VALUES (1,'portaladmin');
UPDATE UserAttributes SET commonName = 'portaladmin', privilegeName = 'Portal Administrator', date_modified = NOW(), addr_modified = '127.0.0.1' WHERE user_id = 1;

# inserting default portal admin pass

INSERT INTO Attribute VALUES (1, 0, 23, 1, 'wisebed', NULL, '127.0.0.1', NULL, NULL);

# --------------------------------------------------------
# ANWANDER 
# --------------------------------------------------------

# --------------------------------------------------------
#
# Table structure for table 'Action'
#

DROP TABLE IF EXISTS Action;

CREATE TABLE Action (
  action_id INT NOT NULL AUTO_INCREMENT,
  role_user INT NOT NULL,
  action_name VARCHAR(40) NOT NULL,
  action_title VARCHAR (255) NOT NULL,
  action_description TEXT NULL,
  date_created TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
  date_modified TIMESTAMP NULL,
  PRIMARY KEY (action_id)
);


# --------------------------------------------------------
#
# Table structure for table 'ActionManager'
#

DROP TABLE IF EXISTS ActionManager;

CREATE TABLE ActionManager (
  action_id INT NOT NULL,
  role_id INT NOT NULL,
  PRIMARY KEY (action_id,role_id)
);


# --------------------------------------------------------
#
# Predefined Actions
#
INSERT INTO `Action` VALUES (1,1,'makingOwnReservation','Create, modify or remove own reservations','Can create, modify or remove own reservations.','2010-04-06 12:02:46',NULL),(2,1,'maintainReservation','Maintaing all reservation','Admin state. Can modify all reservations.','2010-04-06 12:02:46',NULL),(3,1,'maintainTARWIS','Maintaing TARWIS','Admin state. Can modify all databases and ...','2010-04-06 12:02:46',NULL),(4,1,'getReservations','Access the rervation table','Can access the rervation table','2010-04-06 12:02:46',NULL),(5,1,'createExperiment','Define an experiement','Can define an experiement','2010-04-06 12:02:46',NULL),(6,1,'viewNetwork','View an all nodes','Can see all nodes','2010-04-06 12:02:46',NULL),(7,4,'makeReservation','make a Reservation','','2010-04-22 08:53:16',NULL),(11,13,'deleteReservation','delete a reservation','','2010-06-14 14:05:27',NULL);

# --------------------------------------------------------
#
# Predefined Roles
#
INSERT INTO `Role` VALUES (1,'1.E10F7CBD7C',1,2,1,'TARWIS User','http://www.wisebed.eu','simple-redirect','','2010-04-06 12:06:14','130.92.65.74','2010-06-21 08:54:03','130.92.65.74'),(2,'2.42E359CC94',1,2,1,'TARWIS Admin','http://www.wisebed.eu','simple-redirect','','2010-04-06 12:06:54','130.92.65.74','2010-06-21 08:54:13','130.92.65.74'),(3,'3.7AB7716F2E',1,2,1,'TARWIS Visitor','http://www.wisebed.eu','simple-redirect','','2010-04-06 12:07:22','130.92.65.74','2010-04-26 12:46:47','130.92.65.70');

# --------------------------------------------------------
#
# Predefined Action <-> Role associations
#
INSERT INTO `ActionManager` VALUES (1,1),(1,2),(2,2),(3,2),(4,1),(4,2),(5,1),(5,2),(6,1),(6,2),(6,3),(7,1),(7,2),(11,1),(11,2);



