<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Adapting.class.inc,v 1.10 2006/09/22 15:44:33 vlry Exp $
 */
/* ****************************************************************** */

/**
 * Load the adapters configuration 
 */
require_once("config/adapters.config");

/**
 * Adapting is an utility class which provides static 
 * utility functions for Role Adaptors.  
 *
 * 
 * @author Aljoscha Bachmayer <bachmaye@iam.unibe.ch>
 * @version $Revision: 1.10 $
 * @package Library
 *
 * @abstract 
 */ 
class Adapting {


/* **************************************************************** */
/**
 * Creates a password with small letters, capitals and digits for a foreign Role
 *
 * @param int $length  the length of the password
 *
 * @return string the created password 
 *
 * @static
 */
function createPassword($length) {
  $tmp="";
  for ($i=0; $i<$length; $i++) {
	$cap_let_dig=rand(0,2);
	switch ($cap_let_dig) {
	case 0:
	    $tmp=$tmp.chr(rand(97,122));
	    // appends a small letter
	    break;
	case 1:
	    $tmp=$tmp.chr(rand(65,90));
	    // appends a capital
	    break;
	case 2:
	    $tmp=$tmp.chr(rand(48,57));
	    // appends  a digit
	    break;
	}
  }
  return $tmp;  
}

/* **************************************************************** */
/**
 * Creates a username with small letters, capitals and digits for a foreign Role
 *
 * @param int $length  the length of the username
 *
 * @return string the created username 
 *
 * @static
 */
function createUsername($length) { 
  return Adapting::create_password($length);
}


/* **************************************************************** */
/**
 * Creates a MAC Hash based on a shared key
 *
 * @param string $key  the shared key
 *
 * @param string $data data to be encrypted
 *
 * @static
 */
function createMACHash($key, $data) { 
  $b = 64; // byte length for md5
  if (strlen($key) > $b) {
      $key = md5($key);
  }
  $key  = str_pad($key, $b, chr(0x00));
  $ipad = str_pad('', $b, chr(0x36));
  $opad = str_pad('', $b, chr(0x5c));
  $k_ipad = $key ^ $ipad ;
  $k_opad = $key ^ $opad;
  
  return md5($k_opad . md5($k_ipad . $data));

}


/* **************************************************************** */
/**
 * Lists all Role Adaptors
 *
 * @return array 2-Dimensional Array with name and display-name of all Role Adaptors
 *
 * @static
 */
function listRoleAdaptors() { 
  global $ADAPTING_ADAPTER_CONFIG;

  $ret = array(); 
  if (! isset($ADAPTING_ADAPTER_CONFIG)) {
     return $ret; 
  }  

  foreach($ADAPTING_ADAPTER_CONFIG as $adapter_id => $adapter_config) {
    $ret[$adapter_id] = array(
       "name" => "$adapter_id",
       "display-name" => $adapter_config["display-name"]
    );
  }
  return $ret;
}


/* **************************************************************** */
/**
 * Gets the Help to a given Role Adaptor
 *
 * @param string $adaptor_name  name of the Role Adaptor
 * 
 * @return string path to Helpfile
 *
 * @static
 */
function getAdapterHelp($adapter_name) { 
  global $ADAPTING_ADAPTER_CONFIG;

  if (! isset($ADAPTING_ADAPTER_CONFIG)) {
     return false; 
  }
  if (! isset($ADAPTING_ADAPTER_CONFIG[$adapter_name])) {
     return false; 
  }
  $param = $ADAPTING_ADAPTER_CONFIG[$adapter_name];
  if (! isset($param["help-text"])) {
    return false; 
  }
  
  return $param["help-text"];
}

/* **************************************************************** */
/**
 * lists the Adaptor's additional Parameters
 *
 * @param string $path  $adaptor_name  name of the Role Adaptor
 *
 * @return array all names of parameters
 *
 * @static
 */
function getAdapterParameterDescriptors($adapter_name) { 
 global $ADAPTING_ADAPTER_CONFIG;
  $mynull= null;
  if (! isset($ADAPTING_ADAPTER_CONFIG)) {
     return $mynull;
  }
  if (! isset($ADAPTING_ADAPTER_CONFIG[$adapter_name])) {
     return $mynull;
  }
  $param = $ADAPTING_ADAPTER_CONFIG[$adapter_name];
  if (! isset($param["parameters"])) {
    return $mynull;
  }
  return $param["parameters"];
}

/* **************************************************************** */
/**
 * Replies the name of the adapter PHP file (relative to 
 * {@link PORTAL_BASE_URL}.
 *
 * @param string $adapterName  the name of the adapter
 *
 * @return string the name of the adapter PHP file; false, if 
 *   there is no such adapter or the adapter file name is
 *   undefined
 */
function getAdapterFileName($adapterName) {
 global $ADAPTING_ADAPTER_CONFIG;

  if (! isset($ADAPTING_ADAPTER_CONFIG)) {
     return false;
  }
  if (! isset($ADAPTING_ADAPTER_CONFIG[$adapterName])) {
     return false;
  }
  $param = $ADAPTING_ADAPTER_CONFIG[$adapterName];
  if (! isset($param["adapter-file"])) {
    return false;
  }
  return $param["adapter-file"];
}

/**
 * Replies the display name of the given adapter.
 *
 * @param string $adapter  the name of the adapter
 *
 * @return string the display name of the adapter or NULL.
 * @static
 */
function getAdapterDisplayName($adapter) {
 global $ADAPTING_ADAPTER_CONFIG;

  $mynull= null;
  if (! isset($ADAPTING_ADAPTER_CONFIG)) {
     return $mynull;
  }
  if (! isset($ADAPTING_ADAPTER_CONFIG[$adapter])) {
     return $mynull;
  }
  $params= $ADAPTING_ADAPTER_CONFIG[$adapter];
  if (! isset($params["display-name"])) {
    return false;
  }
  return $params["display-name"];
}


/* **************************************************************** */
/**
 * Gets all User Attributes
 *
 * @param string $uid  User-Id
 *
 * @return array an array with Attributename, Attributevalue and whether the value is trusted (in this order)
 *
 * @static
 */
function getUserAttributes($uid) { 
  $conn  =& MySQLConnection::getConnection();
  $query =	" SELECT attributetype_name,"
		. " attribute_value,"
		. " attribute_trusted"
		. " FROM Attribute, AttributeType, User"
		. " where attributetype_id=attribute_type"
		. " and attribute_user=user_id"
		. " and user_uid='".$uid."'";

        

  $conn->query($query);
  $ret = array();
  $i = 0;
  while($row = $conn->fetch_array()) {
     $tmp=array();
     $tmp[0]=$row[0];
     $tmp[1]=$row[1];
     $tmp[2]=$row[2];
     $ret[$i] = $tmp;
     $i++;
  }
  $conn->freeresult();
  return $ret;
  $conn->close();
}

}

?>
