<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Messaging.class.inc,v 1.34 2007/05/10 09:30:09 vlry Exp $
 */


/**
 * Messaging class provides methods for sending out notification mails
 * and sms.
 *
 * @version $Revision: 1.34 $
 * @package Library 
 *
 */ 


/* Mail class is required */
require_once("include/PHPMailer.class.inc");
require_once("include/Logger.class.inc");

/** Constants representing predefined status changed messages */
define('NEW_SUBSCRIPTION', "new_subscription");
define('CLOSE_SUBSCRIPTION', "close_subscription");
define('SUBSCRIPTION_CANCELLED', "subscription_cancelled");
define('SUBSCRIPTION_ACCEPTED', "subscription_accepted");
define('SUBSCRIPTION_REJECTED', "subscription_rejected");
define('SUBSCRIPTION_SUSPENDED', "subscription_suspended");
define('SUBSCRIPTION_DELETED', "subscription_deleted");
/* default subject for notification emails */
define('NOTIFICATION_SUBJECT', "default_subject");



class Messaging {

    var $error= NULL;
    var $enabled;
    var $SMSGateway;
    var $maxSMSLength;
    var $mailer= 'smtp'; // smtp, mail or sendmail
    

    /**
    * Default constructor
    */
    function Messaging() {

        /* check if notification system is set to enabled */
        $this->enabled = false;
        if (defined("ENABLE_NOTIFICATION")) {
            if (ENABLE_NOTIFICATION === true) {
                $this->enabled = true;
            }
        }

        /* maximum SMS body length */
        $this->maxSMSLength = 120;
        if (defined("MAX_SMS_LENGTH")) {
                $this->maxSMSLength = MAX_SMS_LENGTH;
        }

        /* set SMSGateway to config value */
        $this->SMSGateway = false;
        if (defined("USE_SMSGATEWAY")) {
            if (USE_SMSGATEWAY === true) {
                if (defined("SMSGATEWAY"))  {
                    $this->SMSGateway = SMSGATEWAY;
                }
            }
        }

        $this->mailer= 'smtp';
        if (defined('AAI_PORTAL_SMTP')) {
            if ( AAI_PORTAL_SMTP == 'mail') {
                $this->mailer= 'mail';
            }
            else if (AAI_PORTAL_SMTP == 'sendmail') {
                $this->mailer= 'sendmail';
            }
            else if (AAI_PORTAL_SMTP == 'qmail') {
                $this->mailer= 'qmail';
            }
            else {
                // default: use SMTP server
                $this->mailer= 'smtp';
            }
        }

        /* get current locale and load notification messages */
        I18N::initLocale();
        I18N::loadMessageBundle("notification/state-change");

    }

    /**
    * sends a notification message to set of email recepients
    *
    * @param string $from           email address of sender (role administator)
    * @param string $to             comma separated list of email recepients
    * @param string $subject        email subject
    * @param string $body           email message body
    * @param boolean $useBcc        set true if you want recipients to be send as Bcc
    * @param string $fromName       name of sender
    *
    * @return boolean if mail was sent out successfully
    */
    function sendAsMail($from, $to, $subject, $body, $useBcc= false, $fromName= null) {

        // convert comma sep list to array
        $to_list= explode(',', $to);

        // use SMTP PHPMailer
        $mail= new PHPMailer();
        // options: from where to load class.smtp.php
        $mail->PluginDir= 'include/';
        $mail->SetLanguage('en', 'include/');

        $mail->AddCustomHeader("X-SNAportal: " . SNA_PORTAL_VERSION );

        // SMTP host or mail/sendmail
        // default for PHPMailer is PHP mail()
        if ( $this->mailer == 'smtp' ) {
            $mail->IsSMTP(); // set mailer to use SMTP
            $mail->Host= AAI_PORTAL_SMTP; // SMTP hostname from sna-portal.config
        }
        else if ($this->mailer == 'mail') {
            $mail->IsMail();
        }
        else if ($this->mailer == 'qmail') {
            $mail->IsQmail();
        }
        else if ($this->mailer == 'sendmail') {
            $mail->IsSendmail();
        }
        
        // From and Reply headers
        $mail->From= $from;
        $mail->Sender= $from;
        if (is_null($fromName)) {
            $mail->FromName= "SNAportal";
        }
        else {
            $mail->FromName= "SNAportal $fromName";
        }
        // SMS Gateway specification for error
        // $mail->AddReplyTo( $from ); // SMS delivery notification
        $mail->AddCustomHeader( "Errors-To: $from" ); // SMS error notification

        // To or BCC ? use Bcc header if recepients shall be hidden
        if ($useBcc) {          
            foreach( $to_list as $address ) {
                $mail->AddBCC( $address );
            }
        }
        else {
            foreach( $to_list as $address ) {
               $mail->AddAddress( $address );
            }
        }

        // Subject and Body
        $mail->Subject= $subject;
        $mail->Body= $body;

        // send
        $ok= $mail->Send();
        $this->error= $mail->ErrorInfo;
        if (!empty($this->error)) {
            $logger=& Logger::getInstance();
            $logger->error("Messaging::sendAsMail: " . $this->error);
        }

        return $ok;

    }


    /**
    * sends a notification message to set of mobile numbers via a sms gateway
    *
    * @param string $from           email address of sender (role administator)
    * @param string $to             comma separated list of mobile number recepients
    * @param string $body           email message body
    *
    * @return boolean if mail was sent out successfully
    */
    function sendAsSMS($from, $to, $body) {

        /* return value */
        $ok= false;

        if ((strlen($body) <= $this->maxSMSLength) && ($this->SMSGateway !== false)) {
            // convert comma sep list to array
            $to_list= explode(',', $to);
            $mobiles= array();
            foreach ($to_list as $mobile) { 
                $mobile= $this->filter_mobile( $mobile );
                $mobiles[] = str_replace("<mobile>", $mobile, $this->SMSGateway);
            }
            // convert array to comma sep string
            $rcpts= implode(',', $mobiles);   
            /* continue as it were a normal email message */
            $ok = $this->sendAsMail($from, $rcpts, '', $body);
        
        } 
        else {
            $logger=& Logger::getInstance();
            if (strlen($body) > $this->maxSMSLength) { 
                $logger->warn("Messaging::sendAsSMS: message body is too long.");
            }
            else {
                $logger->warn("Messaging::sendAsSMS: SMSGateway is not defined.");            
            }
        }

        return $ok;

    }

    /**
     * Filters the mobile telephone number. 
     *
     * Replace first '+' with '00' and then suppress all 
     * non-digit characters.
     *
     * @param string $mobile The mobile number.
     * @return string The filtered mobile number.
     */
    function filter_mobile($mobile) {
        $mobile= trim($mobile);
        $mobile= str_replace('+', '00', $mobile);
        $mobile= ereg_replace("[^[:digit:]]", "", $mobile);
        return $mobile;
    }


    /**
    * sends a automatic notification message containing a predefined text
    *
    * @param string $from  Email address of sender (role administator).
    * @param string $to    Comma separated list of recepient email addresses.
    * @param int $msgId    id of notification message, see constants NOTIFICATION_*. 
    * @param boolean $useBcc If true, send as bcc.
    * @param array $data   Hashtable with additional data for the email body:
    *                      {'from_name' => X, 'to_name' => Y, 'role_name' => Z }
    *
    * @return boolean if mail was sent out successfully
    */
    function stateChanged($from, $to, $msgId, $useBcc= false, $data= array() ) {

        /* return value */
        $ok = false;
        $subject = $this->getDefaultSubject();
        $fromName= null;
        if (isset($data['from_name'])) {
            // email from address: SNAportal (User|Admin): <commonName>
            switch($msgId) {
                case NEW_SUBSCRIPTION:
                case CLOSE_SUBSCRIPTION:
                    $fromName= "User: {$data['from_name']}"; 
                    break;
                default:
                    $fromName= "Admin: {$data['from_name']}"; 
                    break;
            }
        }
        if (isset($data['from_name']) and isset($data['to_name']) and isset($data['role_name'])) {
            switch($msgId) {
                case NEW_SUBSCRIPTION:
                case CLOSE_SUBSCRIPTION:
                    // notification sent to the admins, user name: <commonName> [ID: <userid>]
                    if ( isset($data['from_uid']) ) {
                        $data['from_name']= "{$data['from_name']} [ID: {$data['from_uid']}]";                     
                    }
                    break;
                default:
                    // notification sent to the user, admin name: <commonName> (<email>)
                    $data['from_name']= "{$data['from_name']} ($from)"; 
                    break;
            }
            if ( ! isset($data['role_key']) ) {
                $data['role_key']= null;
            }
            
            $body= I18N::getMessage("state-change-2.$msgId",
                                    $data['from_name'], $data['to_name'],
                                    $data['role_name'], 
                                    PORTAL_BASE_URL,
                                    SUPPORT_URL, SUPPORT_EMAIL,
                                    $data['role_key']
                                    );
        }
        else {
            $body= I18N::getMessage("state-change.$msgId");
        }
        
        $ok= $this->sendAsMail($from, $to, $subject, $body, $useBcc, $fromName);
        return $ok;

    }

    /**
    * returns the default notification subject
    *
    * @return string default localized subject
    */
    function getDefaultSubject() {
        return I18N::getMessage("state-change." . NOTIFICATION_SUBJECT);
    }

    /**
    * returns the maximum sms length
    *
    * @return int maximum sms length
    */
    function getMaxSMSLength() {
        return $this->maxSMSLength;
    }

    /**
    * returns if the messaging system is enabled or not
    *
    * @return boolean returns true if notification system is enabled, otherwise false.
    */
    function is_enabled() {
        return $this->enabled;
    }
    
    /**
     * Returns the underlying error message (PHPMailer).
     *
     * @return String The underlying error message or NULL.
     */
    function getError() {
        return $this->error;
    }

}

?>
