<?php

/* ****************************************************************** */
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003  Swiss Virtual Campus (http:/*www.swissvirtualcampus.ch) 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
* $Id: Vitels.class.inc,v 1.1 2004/03/16 16:54:44 tspreng Exp $
*/
/* ****************************************************************** */

/**
* Vitels is a class that provides static functions for adapters which
* will use to VITELS roles.
*
* @version  $Revision: 1.1 $
* @package Library
*/


class Vitels {
   /* =============================
      member variables
      ============================= */
   
   var $server;
   var $port;
   var $bind_dn;
   var $bind_pw;
   var $base_dn;
   var $aai_base_dn;
   var $pwd_attribute;
   var $cookie_name;
   var $cookie_domain;
   var $cookie_path;
   var $cookie_expires;
   var $res;


   function Vitels() {
	  /* default values */
      /* Vitels LDAP server settings */
      $this->server     = "localhost";
      $this->port       = "389";
      
      /* LDAP bind settings
         VITELS LDAP administrator must provide these credentials */
      $this->bind_dn    = "uid=admin,ou=Staff,o=VITELS,c=CH";
      $this->bind_pw    = "somepassword";
      
      /* BaseDN settings
         if the domain of the user matches any of these keys
         then the corresponding base dn will be used to store their data.
         The following is an example where the users of different universities
         are stored in different subtrees */
      /*   $this->base_dn = array("unibe" => "ou=users,o=Universitaet Bern,c=CH",
                         "unifr" => "ou=users,o=Universite-de-Fribourg,c=CH",
                         "unige" => "ou=users,o=Universite-de-Geneve,c=CH",
                         "unine" => "ou=users,o=Universite-de-Neuchatel,c=CH",
                         "ingfr" => "ou=users,o=Ecole-Ingenieurs-Fribourg,c=CH"); */
      
      /* by default just store all users under the following subtree */
      $this->base_dn = array("portal" => "ou=users,o=SNA-Portal,c=CH");
      
      /* base DN for users whose domain does not match any of the $this->base_dn keys */
      $this->aai_base_dn = $this->base_dn["portal"];
      
      /* attribute name for writing aai password */
      $this->pwd_attribute = "userPassword";
      
      /* name, domain and path scope of the authentication cookie */
      $this->cookie_name = "aaivitels";
      $this->cookie_domain = $_SERVER["HTTP_HOST"];
      $this->cookie_path = "/";
      $this->cookie_expires = 0;
   }
   
   /* =============================
      member functions
      ============================= */
   
   function prepareAndWriteCookie($customParams, $aai_uid, $pwd) {
   
   	/* prepare cookie */
       $this->cookie_data = base64_encode($aai_uid.":".$pwd);
       if (isset($customParams["cookie-domain"]) && !empty($customParams["cookie-domain"])) {
           $this->cookie_domain = $customParams["cookie-domain"];
       }
       if (isset($customParams["cookie-path"]) && !empty($customParams["cookie-path"])) {
           $this->cookie_path = $customParams["cookie-path"];
       }
       if (isset($customParams["cookie-expires"]) && !empty($customParams["cookie-expires"])) {
           if (is_numeric($customParams["cookie-expires"])) {
               $this->cookie_expires = time() + $customParams["cookie-expires"];
           }
       }
   
       /* set the authentication cookie */
       setcookie($this->cookie_name, $this->cookie_data, $this->cookie_expires, $this->cookie_path, $this->cookie_domain);

   }
   
   function rejectLocalUser($customParams, $aai_uid) {
   	/* do not forward a user with a local uid to an AAI protected VITELS page */
   	if (!preg_match("/@/", $aai_uid)) {
   	    /* seems like we have found a local user */
   	    if (!(trim($customParams["cookie-based"]) == "1")) {
   	        $this->printErrorPage("local users cannot be redirected to an AAI protected VITELS page!");
   	    }
   	}
   }						
   
   function setCustomParams($customParams) {
   
   	/* get custom VITEKS LDAP parameters */
   	if (isset($customParams["vitels-ldap-host"]) && !empty($customParams["vitels-ldap-host"])) {
   	    $this->server = $customParams["vitels-ldap-host"];
   	}
   	if (isset($customParams["vitels-ldap-port"]) && !empty($customParams["vitels-ldap-port"])) {
   	    $this->port = $customParams["vitels-ldap-port"];
   	}
   	if (isset($customParams["vitels-bind-dn"]) && !empty($customParams["vitels-bind-dn"])) {
   	    $this->bind_dn = $customParams["vitels-bind-dn"];
   	}
   	if (isset($customParams["vitels-bind-pw"]) && !empty($customParams["vitels-bind-pw"])) {
   	    $this->bind_pw = $customParams["vitels-bind-pw"];
   	}
   	if (isset($customParams["vitels-aaiuser-base-dn"]) && !empty($customParams["vitels-aaiuser-base-dn"])) {
   	    $this->aai_base_dn = $customParams["vitels-aaiuser-base-dn"];
   	}
   }
   
   function LDAPConnect() {
   	$res = ldap_connect($this->server, $this->port);
   	if ($res) {
   		/* connection successful */
   		$bind = ldap_bind($res, $this->bind_dn, $this->bind_pw);
   		if (!$bind) {
   			/* bind failed */
   			$this->printErrorPage(ldap_error($res));
   		}
   	} else {
   		/* cant connect */
   		$this->printErrorPage("Unable to connect to LDAP server");
   	}
   	$this->res = $res;
   }
   
   function closeLDAP() {
      ldap_close($this->res);
   }	  
   
   function addToLDAP($uid) {
   	
   	list ($userid, $domain) = $this->splitUid($uid);
   	if (array_key_exists($domain, $this->base_dn)) {
   		$base = $this->base_dn[$domain];
   	} else {
   		$base = $this->aai_base_dn;
   		$userid = $uid;
   	}
   	/* prepare data */
   	$info["cn"]="aai_$uid";
   	$info["sn"]="aai_$uid";
   	$info["uid"]=$userid;
   	$info["objectclass"][0]="account";
   	$info["objectclass"][1]="person";
   
   	/* add data to directory */
   	$ar = ldap_add($this->res, "uid=$userid,$base", $info);
   	if ($ar) {
   		return true;
   	} else {
   		return false;
   	}
   }
   
   
   function setLDAPPasswd($pwd, $uid) {
   
   	list ($userid, $domain) = $this->splitUid($uid);
   	if (array_key_exists($domain, $this->base_dn)) {
   		$base = $this->base_dn[$domain];
   	} else {
   		$base = $this->aai_base_dn;
   		$userid = $uid;
   	}
   	$data[$this->pwd_attribute] = $pwd;
   	ldap_modify($this->res, "uid=$userid,$base", $data);
   }
   		
   
   function checkUid($uid) {
   
   	list ($userid, $domain) = $this->splitUid($uid);
   	if (array_key_exists($domain, $this->base_dn)) {
   		$base = $this->base_dn[$domain];
   	} else {
   		$base = $this->aai_base_dn;
   		$userid = $uid;
   	}
   	$sr = ldap_search($this->res, $base, "uid=$userid");
   	if (ldap_count_entries($this->res, $sr) > 0) {
   		return true;
   	} else {
   		return false;
   	}
   }
   
   function splitUid($uid) {
   	if (preg_match("/@/", $uid)) {
   		/* we have an uid with userid@domain syntax */
   		$parts = preg_split("/@/", $uid);
   		$userid = $parts[0];
   		$domain = $parts[1];
   		/* remove the trailing tld ('.ch') */
   		$domain = preg_replace("/\.\w\w$/", "", $domain);
   	} else {
   		/* we have a uid without domain */
   		$userid = $uid;
   		$domain = "none";
   	}
   	return array($userid, $domain);
   }
   
   
   function printErrorPage($error) {
   	/* sollte noch mehr details hin.... */
   
   	/* include the page header */
   	include("include/header.inc");
   	print "<p>An error occured while trying to automatically log into VITELS.</p>\n";
   	print "<p>please report the folloing error to the responsible person:<br>\n";
   	print "<b>".$error."</b></p>\n";
   	/* include the common footer */
   	include("include/footer.inc");
   	exit;
   }

}	
   	
?>
