<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (c) 2004 SWITCH (htttp://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ShibbolethHTTPHeaders.inc,v 1.13 2006/09/29 08:42:43 vlry Exp $
 */

/**
 * This file provides a collection of Shibboleth constants.
 *
 * @version $Revision: 1.13 $
 * @package Library
 * @subpackage AAI
 */

// ATID_*
require_once('include/constants.inc');
// Logger
require_once('include/Logger.class.inc');

/**
 * Defines all Shibboleth HTTP Headers
 *
 * @see Shibboleth Attributes Acceptantce Policy (AAP.xml)
 */
$SHIBBOLETH_HTTP_HEADERS= array(
  "Shib-WISEBEDPerson-personUniqueID",
  "Shib-WISEBEDPerson-surname",
  "Shib-WISEBEDPerson-givenName",
  "HTTP_SHIB_EP_DATEOFBIRTH",
  "HTTP_SHIB_EP_GENDER",
  "Shib-WISEBEDPerson-preferredLanguage",
  "Shib-WISEBEDPerson-mail",
  "HTTP_SHIB_INETORGPERSON_HOMEPOSTALADDRESS",
  "HTTP_SHIB_ORGPERSON_POSTALADDRESS",
  "HTTP_SHIB_INETORGPERSON_HOMEPHONE",
  "HTTP_SHIB_PERSON_TELEPHONENUMBER",
  "HTTP_SHIB_INETORGPERSON_MOBILE",
  "Shib-WISEBEDPerson-homeOrganization",
  "Shib-WISEBEDPerson-homeOrganizationType",
  "Shib-WISEBEDPerson-eduPersonAffiliation",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH1",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH2",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH3",
  "HTTP_SHIB_EDUPERSONSTUDYLEVEL",
  "HTTP_SHIB_EDUPERSONSTAFFCATEGORY",
  "HTTP_SHIB_EP_ORGDN",
  "HTTP_SHIB_EP_ORGUNITDN",
  "Shib-EP-Entitlement"
);


/**
 * Shibboleth HTTP headers to SESSION mapping.
 */
$SHIBBOLETH_HTTP_HEADER_SESSION_MAPPING= array(
  "Shib-WISEBEDPerson-personUniqueID"          => "aai_Shib-WISEBEDPerson-personUniqueID",
  "Shib-WISEBEDPerson-surname"            => "aai_surname",
  "Shib-WISEBEDPerson-givenName"   => "aai_givenName",
  "HTTP_SHIB_EP_DATEOFBIRTH"       => "aai_eduPersonBirthdate",
  "HTTP_SHIB_EP_GENDER"            => "aai_eduPersonGender",
  "Shib-WISEBEDPerson-preferredLanguage" => "aai_preferredLanguage",
  "Shib-WISEBEDPerson-mail"        => "aai_mail",
  "HTTP_SHIB_INETORGPERSON_HOMEPOSTALADDRESS" => "aai_homePostalAddress",
  "HTTP_SHIB_ORGPERSON_POSTALADDRESS"   => "aai_postalAddress",
  "HTTP_SHIB_INETORGPERSON_HOMEPHONE"   => "aai_homePhone",
  "HTTP_SHIB_PERSON_TELEPHONENUMBER"    => "aai_telephoneNumber",
  "HTTP_SHIB_INETORGPERSON_MOBILE"      => "aai_mobileTelephoneNumber",
  "Shib-WISEBEDPerson-homeOrganization"  => "aai_Shib-WISEBEDPerson-homeOrganization",
  "Shib-WISEBEDPerson-homeOrganizationType" => "aai_Shib-WISEBEDPerson-homeOrganizationType",
  "Shib-WISEBEDPerson-eduPersonAffiliation"            => "aai_eduPersonAffiliation",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH1"      => "aai_eduPersonStudyBranch1",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH2"      => "aai_eduPersonStudyBranch2",
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH3"      => "aai_eduPersonStudyBranch3",
  "HTTP_SHIB_EDUPERSONSTUDYLEVEL"        => "aai_eduPersonStudyLevel",
  "HTTP_SHIB_EDUPERSONSTAFFCATEGORY"     => "aai_eduPersonStaffCategory",
  "HTTP_SHIB_EP_ORGDN"                  => "aai_eduPersonOrgDN",
  "HTTP_SHIB_EP_ORGUNITDN"              => "aai_eduPersonOrgUnitDN",
  "Shib-EP-Entitlement"            => "aai_eduPersonEntitlement"
);

/**
 * Shibboleth HTTP headers to AttributeTypeID (ATID) mapping.
 */
$SHIBBOLETH_HTTP_HEADER_ATID_MAPPING= array(
  "Shib-WISEBEDPerson-personUniqueID"          => ATID_EDUPERSONUNIQUEID,
  "Shib-WISEBEDPerson-surname"            => ATID_SURNAME,
  "Shib-WISEBEDPerson-givenName"   => ATID_GIVENNAME,
  "HTTP_SHIB_EP_DATEOFBIRTH"       => ATID_EDUPERSONBIRTHDATE,
  "HTTP_SHIB_EP_GENDER"            => ATID_EDUPERSONGENDER,
  "Shib-WISEBEDPerson-preferredLanguage" => ATID_PREFERREDLANGUAGE,
  "Shib-WISEBEDPerson-mail"        => ATID_MAIL,
  "HTTP_SHIB_INETORGPERSON_HOMEPOSTALADDRESS" => ATID_HOMEPOSTALADDRESS,
  "HTTP_SHIB_ORGPERSON_POSTALADDRESS"   => ATID_POSTALADDRESS,
  "HTTP_SHIB_INETORGPERSON_HOMEPHONE"   => ATID_HOMEPHONE,
  "HTTP_SHIB_PERSON_TELEPHONENUMBER"    => ATID_TELEPHONENUMBER,
  "HTTP_SHIB_INETORGPERSON_MOBILE"      => ATID_MOBILETELEPHONENUMBER,
  "Shib-WISEBEDPerson-homeOrganization"  => ATID_EDUPERSONHOMEORGANIZATION,
  "Shib-WISEBEDPerson-homeOrganizationType" => ATID_EDUPERSONHOMEORGANIZATIONTYPE,
  "Shib-WISEBEDPerson-eduPersonAffiliation"            => ATID_EDUPERSONAFFILIATION,
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH1"      => ATID_EDUPERSONSTUDYBRANCH1,
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH2"      => ATID_EDUPERSONSTUDYBRANCH2,
  "HTTP_SHIB_EDUPERSONSTUDYBRANCH3"      => ATID_EDUPERSONSTUDYBRANCH3,
  "HTTP_SHIB_EDUPERSONSTUDYLEVEL"        => ATID_EDUPERSONSTUDYLEVEL,
  "HTTP_SHIB_EDUPERSONSTAFFCATEGORY"     => ATID_EDUPERSONSTAFFCATEGORY,
  "HTTP_SHIB_EP_ORGDN"                  => ATID_EDUPERSONORGDN,
  "HTTP_SHIB_EP_ORGUNITDN"              => ATID_EDUPERSONORGUNITDN,
  "Shib-EP-Entitlement"            => ATID_EDUPERSONENTITLEMENT
);


/**
 * Helpers class
 */
class ShibbolethHTTPHeaders {

    /**
     * Copies the values of the Shibboleth attributes from HTTP headers to 
     * matching slots in the global session object $_SESSION.
     *
     * @see $SHIBBOLETH_HTTP_HEADER_SESSION_MAPPING
     * @see global $_SESSION
     * @static
     */
    function storeInSession() {
        global $SHIBBOLETH_HTTP_HEADER_SESSION_MAPPING;
        foreach($SHIBBOLETH_HTTP_HEADER_SESSION_MAPPING as $http_key => $session_key) {
            if (isset($_SERVER[$http_key]) and !empty($_SERVER[$http_key]) ) {
                // shibboleth attribute value are UTF-8 encoded
                $value= utf8_decode( $_SERVER[$http_key] );
                $_SESSION[$session_key]= $value;
            }
        }
    }
    
    /**
     * Stores the Shibboleth attributes from HTTP headers as user 
     * trusted attributes. 
     *
     * @param $user The user to update (User class)
     *
     * @see $SHIBBOLETH_HTTP_HEADER_ATID_MAPPING
     * @static
     */    
    function updateAAIAttributes($user) {
        global $SHIBBOLETH_HTTP_HEADER_ATID_MAPPING;

        // reset all trusted state
        //$user->resetAttributeTrustedState();
        
        // storing attributes provided by an AAI HO in the portal database
        foreach($SHIBBOLETH_HTTP_HEADER_ATID_MAPPING as $http_key => $atid) {
            if ( isset($_SERVER[$http_key]) and !empty($_SERVER[$http_key]) ) {
                // shibboleth attribute value are UTF-8 encoded
                $value= utf8_decode( $_SERVER[$http_key] );
                // don't store Shib-WISEBEDPerson-personUniqueID!
                if ($atid != ATID_EDUPERSONUNIQUEID) {
                    if ($atid == ATID_GIVENNAME) {
                        // ETHZ send multi valued givenName, grrrrr!
                        $pos= strpos($value,';');
                        if ($pos) {
                            // only get first value
                            $value= substr($value,0,$pos); 
                        }
                    }
                    if ($atid == ATID_PREFERREDLANGUAGE) {
                        // UniBE store preferredLanguage in capital 
                        $value= strtolower($value);
                    }
                
                    $logger=& Logger::getInstance();
                    $logger->debug("storeAttribute( $atid, $value, true)");
                    // store as trusted
                    $user->storeAttribute( $atid, $value, true );
                }
            }  
        }
    }

}
?>
