<?PHP
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaAdapter.inc,v 1.7 2006/09/07 15:06:30 vlry Exp $
*/

/** Default request path for WEbCT Vista AS and IMS adapters */
define('WEBCTVISTA_DEFAULT_AS_REQUEST_PATH','/webct/public/autosignon');
define('WEBCTVISTA_DEFAULT_IMS_REQUEST_PATH','/webct/systemIntegrationApi.dowebct');

/** Default path for MyWebCT entry page */
define('WEBCTVISTA_DEFAULT_MYWEBCT_PATH','/webct/viewMyWebCT.dowebct');
/** Default path for course entry page */
define('WEBCTVISTA_DEFAULT_COURSE_PATH','/webct/startFrameSet.dowebct?forward=studentCourseView.dowebct&lcid=');

/** Default IMS enterprise.properties.datasource */
define('WEBCTVISTA_DEFAULT_IMS_ENTERPRISE_SOURCE', 'SNAportal');
/** Default IMS person.sourcedid.source */
define('WEBCTVISTA_DEFAULT_IMS_PERSON_SOURCE', 'WISEBEDaai');
/** Default IMS membership.sourcedid.source */
define('WEBCTVISTA_DEFAULT_IMS_MEMBERSHIP_SOURCE', 'WebCT');
/** Default IMS membership.member.sourcedid.source == person.sourcedid.source */
define('WEBCTVISTA_DEFAULT_IMS_MEMBER_SOURCE', 'WISEBEDaai');


/** Default WebCTVista WebCTID Factory */
define('WEBCTID_DEFAULT_FACTORY', 'WebCTVista');


/**
 * The abstract WebCTVistaAdapter is use to handle the login and register request.
 *
 * @version $Revision: 1.7 $
 * @author Valery Tschopp - SWITCH
 *
 * @package WebCTAPI
 * @abstract
 */
class WebCTVistaAdapter {
    /** glcId of target institution. */
    var $glcid_;
    /** share secret for Standard API adapter */
    var $standard_secret_;
    /** share secret for IMS API adapter */
    var $ims_secret_;
    /** base server url */
    var $server_url_;
        
    /**
    * Constructor: Creates a new WebCT Vista Adapter.
    * The server base URL is given without path extention in format: 
    * <code>http[s]://<host>:<port></code>
    * 
    * @param string $server_url The WebCT Vista server base URL.
    * @param string $glcid The target Institution glcId.
    * @param string $standard_secret The shared secret for Standard API adapter.
    * @param string $ims_secret The shared secret for IMS API adapter.
    */
    function WebCTVistaAdapter($server_url, $glcid, $standard_secret= null, $ims_secret= null) {    
        // abstract class
        if ( !is_subclass_of($this,"WebCTVistaAdapter") ) {
            trigger_error("Abstract class cannot be instanciate", E_USER_ERROR);
            return;
        }

        $this->server_url_= $server_url;
        $this->glcid_= $glcid;
        $this->standard_secret_= $standard_secret;
        $this->ims_secret_= $ims_secret;
    }

    /**
    * Gets the Institution glcId.
    * @return string The Institution glcId.
    */
    function getGlcId() {
        return $this->glcid_;   
    }
    

    /**
    * Sets the shared secret.
    * @param string $secret The shared secret for Standard API adapter.
    */
    function setStandardSecret( $secret ) {
        $this->standard_secret_= $secret;   
    }

    /**
    * Sets the shared secret.
    * @param string $secret The shared secret for IMS API adapter.
    */
    function setIMSSecret( $secret ) {
        $this->ims_secret_= $secret;   
    }
    
    /**
    * Gets the Standard API shared secret.
    * @return string The shared secret for Standard API adapter.
    */
    function getStandardSecret() {
        return $this->standard_secret_;   
    }

    /**
    * Gets the IMS API shared secret.
    * @return string The shared secret for IMS API adapter.
    */
    function getIMSSecret( ) {
        return $this->ims_secret_;   
    }
    
    /**
    * Returns the WUUI (WebCT Unique User Identifer). 
    * The WUUI is build as:
    * <code>$glcid.$webctid</code>
    *
    * @param string $webctid The WebCT Vista user ID.
    * @return string The WUUI.
    */
    function getWUUI( $webctid ) {
        $wuui= $this->glcid_ . "." . $webctid;
        return $wuui;   
    }
    
    /**
    * Builds and returns the default AS request URL. 
    * This URL is built with the WebCT Vista server base URL 
    * and the WEBCTVISTA_DEFAULT_AS_REQUEST_PATH.
    *
    * @return string The default AutoSignon request URL 
    */
    function getASRequestURL() {
        $req_url= $this->server_url_ . WEBCTVISTA_DEFAULT_AS_REQUEST_PATH;
        return $req_url;
    }

    /**
    * Builds and returns the default AS request URL. 
    * This URL is built with the WebCT Vista server base URL 
    * and the WEBCTVISTA_DEFAULT_IMS_REQUEST_PATH.
    *
    * @return string The default IMS request URL
    */
    function getIMSRequestURL() {
        $req_url= $this->server_url_ . WEBCTVISTA_DEFAULT_IMS_REQUEST_PATH;
        return $req_url;
    }
}


?>
