<?php
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaIMS_IMSDataObjectNode.inc,v 1.3 2006/01/24 12:37:42 vlry Exp $
*/

require_once("include/webct/WebCTIMS_IMSDataObjectNode.class.inc");
require_once("include/pre-php-4.2.inc");

/**
* WebCTVistaIMS_IMSDataObjectNode represents a data node in an IMS data
* object.
*
* @author Valery Tschopp - SWITCH
* @version $Revision: 1.3 $
*
* @package WebCTAPI
*/
class WebCTVistaIMS_IMSDataObjectNode extends WebCTIMS_IMSDataObjectNode {
    
    /* ***************************************************************** */
    /*                    constructor                                    */
    /* ***************************************************************** */
    /**
    * constructor
    *
    * @param string $name  the node name
    * @param array  $attributes  the node attributes
    */
    function WebCTVistaIMS_IMSDataObjectNode($name=false, $value=false, $attributes=NULL) {
        parent::WebCTIMS_IMSDataObjectNode($name, $value, $attributes);
    }
    
    
    /* ***************************************************************** */
    /**
    * Converts the node and its children to an XML representation
    *
    * @param WebCTVistaIMS_XMLWriter $xmlWriter   the XML writer
    *
    * @see toXML()
    */
    function writeXML(&$xmlWriter) {
        if (! is_object($xmlWriter) || ! is_a($xmlWriter, "WebCTVistaIMS_XMLWriter")) {
            trigger_error(
            "WebCTVistaIMS_IMSDataObjectNode::toXML: expected instance of 'WebCTVistaIMS_XMLWriter' as first argument, got '$xmlWriter'",
            E_USER_ERROR
            );
        }
        
        if ($this->_value == false) {
            if ($this->getNumChildren() == 0) {
                // no children, just write the empty tag
                //
                $xmlWriter->writeEmptyTag(
                $this->_name,
                $this->_attributes
                );
            } else {
                // at least one child. Write the open tag, the children, and finally
                // the closing tag
                //
                $xmlWriter->writeOpenTag(
                $this->_name,
                $this->_attributes
                );
                $xmlWriter->incIndentLevel();
                foreach($this->_children as $child) {
                    $child->writeXML(&$xmlWriter);
                }
                $xmlWriter->decIndentLevel();
                $xmlWriter->writeCloseTag(
                $this->_name
                );
            }
        } else {
            // writes an element with text
            //
            $xmlWriter->writeElementWithText(
            $this->_name,
            $this->_value,
            $this->_attributes
            );
        }
    }
    
    /**
    * Replies an XML representation for this node
    *
    * @param boolean $includeHeader  true, if the IMS request header shall
    *   be emitted; false, otherwise
    *
    * @see WebCTIMS_XMLWriter::writeIMSRequestHeader()
    * @see writeXML()
    *
    *
    */
    
    function toXML($includeHeader=false, $xml_encoding= null) {
        $writer = new WebCTVistaIMS_XMLWriter();
        if ($includeHeader) {
            $writer->writeIMSRequestHeader($xml_encoding);
        }
        $this->writeXML(&$writer);
        return $writer->getBuffer();
    }
}

?>
