<?php
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaIMS_XMLWriter.inc,v 1.6 2006/01/24 12:37:42 vlry Exp $
*/

require_once("include/webct/WebCTIMS_XMLWriter.class.inc");

/**
* WebCTVistaIMS_XMLWriter is a utility class for writing a well-formed and
* properly indenten IMS objects as XML stream. Output is written to an
* internal string buffer and can be retrieved using the method
* {@link getBuffer() }.
*
* <b>Typical Usage</b>
* <pre>
*   $writer = new WebCTVistaIMS_XMLWriter();
*
*   $writer->writeIMSRequestHeader();
*
*   $writer->writeOpenTag("enterprise");
*   $writer->incIndentLevel();
*
*   $writer->writeOpenTag("properties", array("lang" => "en"));
*   $writer->incIndentLevel();
*
*   $writer->writeElementWithText("datasource","My Datasource");
*   $writer->writeElementWithText("datetime","2003-08-15");
*
*   $writer->decIndentLevel();
*   $writer->writeCloseTag("properties");
*
*   $writer->decIndentLevel();
*   $writer->writeCloseTag("enterprise");
*
*   echo $writer->getBuffer();
* </pre>
*
* This creates the XML fragment
* <pre>
*  &lt;?xml version="1.0" encoding="ISO-8859-1" &gt;
*  &lt;enterprise&gt;
*    &lt;properties lang="en"&gt;
*      &lt;datasource&gt;My Datasource&lt;/datasource&gt;
*      &lt;datetime&gt;2003-08-15&lt;/datetime&gt;
*    &lt;/properties&gt;
*  &lt;/enterprise&gt;
* </pre>
*
* @version $Revision: 1.6 $
* @author Valery Tschopp - SWITCH
* @package WebCTAPI
*
*/
class WebCTVistaIMS_XMLWriter extends WebCTIMS_XMLWriter {
        
    /* ***************************************************************** */
    /*                    constructor                                    */
    /* ***************************************************************** */
    function WebCTVistaIMS_XMLWriter() {
        parent::WebCTIMS_XMLWriter();   
    }
        
    /**
    * Writes the common header for an IMS object in XML format.
    * The common header consists of the standard XML prolog
    * (&lt;?xml version="1.0" encoding="ISO-8859-1" &gt;)
    *
    * @see WebCTIMS_XMLWriter::writeIMSRequestHeader 
    */
    function writeIMSRequestHeader($xml_encoding= null) {
        if (is_null($xml_encoding)) {
            $this->_appendLine('<?xml version="1.0" encoding="ISO-8859-1"?>');
        }
        else {
            $this->_appendLine("<?xml version=\"1.0\" encoding=\"$xml_encoding\"?>");        
        }
    }
}


?>
