<?PHP
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
*
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004-2006 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaLoginAdapter.inc,v 1.29 2006/09/29 12:53:25 vlry Exp $
*/

//require_once("include/application.inc");
require_once("include/Logger.class.inc");
require_once("include/webct/WebCTVistaAdapter.inc");
require_once("include/webct/WebCTVistaAS.inc");
require_once("include/webct/WebCTVistaIMS.inc");

/**
* The WebCTVistaLoginAdapter process the AutoSignon.
*
* @version $Revision: 1.29 $
* @author Valery Tschopp - SWITCH
*
* @package WebCTAPI
*/
class WebCTVistaLoginAdapter extends WebCTVistaAdapter {

    /** Specific IMS parameters */
    var $ims_params_= array();
    
    /** Error handling */
    var $is_error_= false;
    var $server_message_;
 
    /**
    * Constructor:
    * Creates a new WebCT Vista Login (AutoSignon) Adapter.
    *
    * @param string $server_url The WebCT Vista server basename URL.
    * @param string $glcid The target institution glcId.
    * @param string $standard_secret The Standard API secret.
    * @param string $ims_secret The IMS API secret.
    */
    function WebCTVistaLoginAdapter($server_url, $glcid, $standard_secret, $ims_secret ) {
        parent::WebCTVistaAdapter($server_url, $glcid, $standard_secret, $ims_secret );

    }

    /**
     * Sets specific IMS parameters to be pass to IMS object and structure builders.
     *
     * @see WebCTVistaIMS_SwitchAAIAdapting.inc
     *
     * Possible IMS parameters:
     *  WEBCTVISTA_IMS_ENTERPRISE_SOURCE The IMS enterprise datasource. Default: SNAportal
     *  WEBCTVISTA_IMS_PERSON_SOURCE     The IMS person sourcedid.source. Default: WISEBEDaai
     *  WEBCTVISTA_IMS_MEMBERSHIP_SOURCE The IMS membership sourcedid.source. Default: WebCT
     *  WEBCTVISTA_IMS_MEMBER_SOURCE     The IMS member sourcedid.source. Default: WISEBEDaai
     *  WEBCTVISTA_IMS_OBJECT_RECSTATUS  The IMS person recstatus or member role recstatus.
     *  WEBCTVISTA_IMS_PERSON_EXTENSION_COLUMNS The IMS person extension for custom columns.
     *                                          Parameters is an array( column_name => value, ...)
     *  WEBCTVISTA_IMS_PERSON_EXTENSION_PASSWORD Password for the person.
     *
     * @param string $name The name of the constant.
     * @param string $value The value of the parameter.
     */
    function setIMSParameter($name, $value) {
        $this->ims_params_[$name]= $value;
    }

    /**
     * Returns the Person IMS Source.
     * @return string The Person IMS Source.
     */
    function getPersonIMSSource() {
        if (array_key_exists('WEBCTVISTA_IMS_PERSON_SOURCE', $this->ims_params_)) {
            return $this->ims_params_['WEBCTVISTA_IMS_PERSON_SOURCE'];
        }
        else {
            return WEBCTVISTA_DEFAULT_IMS_PERSON_SOURCE;
        }
    }

    
    /**
    * Sends a redirect to client browser pointing to WebCT Vista AutoSignon handler.
    * After sending a HTTP redirect the function <code>exit</code>.
    *
    * @param string $uniqueid The user AAI UniqueID.
    * @param string $courseid The optional WebCT Vista course ID. 
    *                         If not specified, MyWebCT url is used as default.
    */
    function autosignon($uniqueid, $courseid= null) {
        // get WebCTID for this uniqueid
        $webctid= $this->get_webctid( $uniqueid );
        if (is_null($webctid)) {
            $logger= Logger::getInstance();
            $logger->error("WebCTVistaLoginAdapter:autosignon: Unknown AAI user: $uniqueid, no WebCTID found.");
            // display error
            ErrorHandling::fatalErrorAndExit(
               "AutoSignon Error ($uniqueid)",
               "<p><font color=red>Your AAI UniqueID '$uniqueid' is unknown. No corresponding WebCT ID found.</font></p>"
              ."<p>This means that you are not registered in at least one course. "
              ."Therefore, you are not allowed to access your MyWebCT page on the WebCT "
              ."server: ". $this->server_url_ 
              ."</p>"
              ."<tt>Error: ". $this->getErrorMessage() . "</tt><p/>"
            );
        }

        // get ConsortiaId for this uniqueid
        $consortia_id= $this->get_consortia_id( $webctid );
        if (is_null($consortia_id)) {
            $logger= Logger::getInstance();
            $logger->error("WebCTVistaLoginAdapter:autosignon: Unknown WebCTID: $webctid, no ConsortiaID found.");
            // display error
            ErrorHandling::fatalErrorAndExit(
               "AutoSignon Error ($webctid)",
               "<p><font color=red>Your WebCTID '$webctid' is unknown. No corresponding ConsortiaID found.</font></p>"
              ."<p>This means that you are not registered in at least one course. "
              ."Therefore, you are not allowed to access your MyWebCT page on the WebCT "
              ."server: ". $this->server_url_ 
              ."</p>"
              ."<tt>Error: ". $this->getErrorMessage() . "</tt><p/>"
            );
        }
        //XXX: WebCT Vista 3 (SP5) BUG!!!
        // bug workaround: delete JSESSIONID Vista domain cookie before autologin
        if (isset($_COOKIE['JSESSIONID'])) {
            $value= $_COOKIE['JSESSIONID'];
            header("Set-Cookie: JSESSIONID=$value; expires=Mon, 31 Dec 2001 11:11:11 GMT; path=/; domain=virtualcampus.ch");
        }
        // redirect client browser to autosignon url
        $location= $this->get_autosignon_url($consortia_id, $courseid);
        header( "Location: $location" );
        exit;
    }
    
    /** 
     * Builds the AutoSignon URL. 
     *
     * The URL will be either the direct course entry point or MyWebCT entry point if no
     * course id is given.
     *
     * @param string $consortia_id WebCT Vista 3.0 ConsortiaId for user.
     * @param string $courseid WebCT CE course ID. If null the function returns MyWebCT url.
     * @return string The AutoSignon URL.
     * @private
     */
    function get_autosignon_url( $consortia_id, $courseid= null ) {
        // create new autosignon request
        $as_req= new WebCTVistaAS_Request();
        $as_req_url= $this->getASRequestURL();
        $as_req->setRequestUrl( $as_req_url );
        $glcid= $this->getGlcId();
        $as_req->setGlcId( $glcid );
        $as_secret= $this->getStandardSecret();
        $as_req->setSharedSecret( $as_secret );
        
        // set WebCT ID
        $as_req->setWUUI( $consortia_id );

        // set URL
        $url= null;
        if (is_null($courseid)) {
            // read a default somewhere
            $url= $this->get_mywebct_url();
        }
        else {
            $url= $this->get_course_url( $courseid );
        }
        $as_req->setURL( $url );
        
        // build AS url
        $location= $as_req->buildAutosignonUrl();

        return $location;
    }
        
    /**
    * @return string The default MyWebCT URL.
    * @private
    */
    function get_mywebct_url() {
        return $this->server_url_ . WEBCTVISTA_DEFAULT_MYWEBCT_PATH;
    }
    
    /**
    * Returns the URL for the course given as parameter.
    *
    * @param string $courseid WebCT Vista Course ID long format.
    * @return string The studentCourseView URL.
    * @private
    */
    function get_course_url($courseid) {
        // get only short format: every numeric before dot
        $shortid= $courseid;
        $captured= null;
        if( ereg('^([0-9]+)\..*$', $courseid, $captured) ) {
            $shortid= $captured[1];
        }
        return $this->server_url_ . WEBCTVISTA_DEFAULT_COURSE_PATH . $shortid;      
    }


    /**
    * Returns the WebCTID for the given person IMS ID (AAI Unique ID).
    *
    * @param string $person_ims_id The AAI uniqueID of user.
    * @return string The existing WebCTID or NULL if user doesn't exist or an error occurs.
    * @private
    */
    function get_webctid( $person_ims_id ) {
    
        $ims= new WebCTIMS_API();
        $ims_req_url= $this->getIMSRequestURL();
        $ims->setRequestURL($ims_req_url);
        $ims_secret= $this->getIMSSecret();
        $ims->setSharedAPISecret($ims_secret);
        
        $glcid= $this->getGlcId();
        $person_ims_source= $this->getPersonIMSSource();
        $ims_export_req= 
            WebCTVistaIMS_Request::createExportPersonRequest($glcid,
                                                             $person_ims_id,$person_ims_source);
        $ims_res= $ims->invoke($ims_export_req);
        $content= $ims_res->getServerResponse();
        $this->server_message_= $content;
        //XXX
        $logger= Logger::getInstance();
        $logger->debug("WebCTVistaLoginAdapter:get_webctid:\n$content");
        $datas= array();
        $webctid= null;
        if (ereg('<person>.+<userid>(.+)</userid>.+</person>',$content,$datas)) {
            $webctid= $datas[1];
        }
        else if (!eregi('Business Exception',$content) or 
                 !eregi('Failure exporting person object',$content)) {
            $this->is_error_= true;
            $this->server_message_= "WebCTVistaLoginAdaper:get_webctid: '$content'";
            $logger->error($this->server_message_);
            $webctid= null;
        }
        if (empty($webctid)) {
            // user is unknown
            $logger->notice("WebCTVistaLoginAdaper:get_webctid: Unknown user: $person_ims_id $person_ims_source");
            $webctid= null;
        }
        return $webctid;
    }

    /**
    * Returns the ConsortiaID for a particular WebCTID.
    *
    * @param string $webctid The WebCTID of user (login).
    * @return string The resulting ConsortiaID or NULL if user doesn't exist or an error occurs.
    * @private
    */
    function get_consortia_id($webctid) {
        $glcid= $this->getGlcId();
        $ims_get_req= WebCTVistaIMS_Request::createGetConsortiaIdRequest($glcid,$webctid);
        $ims= new WebCTIMS_API();
        $ims_req_url= $this->getIMSRequestURL();
        $ims->setRequestUrl($ims_req_url);
        $standard_secret= $this->getStandardSecret();
        $ims->setSharedAPISecret($standard_secret);

        $response= $ims->invoke( $ims_get_req );
        $content= $response->getServerResponse();
        //XXX echo "XXX: get_consortia_id($webctid): '$content'";
        $datas= array();
        $consortia_id= null;
        if (!eregi('<consortiaid>(.+)</consortiaid>', $content, $datas)) {
            $this->is_error_= true;
            $this->server_message_= "WebCTVistaLoginAdaper:get_consortia_id: '$content'";
            $logger= Logger::getInstance();
            $logger->error($this->server_message_);
            return $consortia_id; // null
        }
        $consortia_id= $datas[1];
        if (eregi('null',$consortia_id) or empty($consortia_id)) {
            // user is unknown
            $consortia_id= null;
        }
        return $consortia_id;
    }

    /**
    * @return boolean return true if an error occurs while registering the user to the course.
    */
    function isError() {
        return $this->is_error_;   
    }
    
    /**
    * @return string The error message.
    */
    function getErrorMessage() {
        return $this->server_message_;   
    }

    /**
    * @return string The WebCT Vista server response message.
    */
    function getServerMessage() {
        return $this->server_message_;   
    }  

}

?>
