<?php
/*
 * Copyright (C) 2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: edit-role-managers.php,v 1.35 2005/10/21 15:37:48 vlry Exp $
 */


// bootstrap application
//
require_once("include/application.inc");

// check request params
if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing"); 
}

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

$rid= $_REQUEST["rid"];
$role=& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("Invalid role id ($rid) submitted");
}

if (! $admin->canEditRole($role)) {
   ErrorHandling::fatalErrorAndExit("You do not have enough privileges to edit role ($rid)");   
}   

//
// Forms processing
//
if (isset($_REQUEST['do_change_provider'])) {
    $new_provider_id= $_REQUEST['new_provider_id'];
//    echo "XXX: new_provider_id= $new_provider_id<br>";
    $role->setRoleProviderID($new_provider_id);
    $role->persist();
}

if (isset($_REQUEST['do_add_manager'])) {
    $new_manager_id= $_REQUEST['new_manager_id'];
//    echo "XXX: new_manager_id= $new_manager_id<br>";
    $role->addRoleManagerID($new_manager_id);
}

if (isset($_REQUEST['do_remove_manager'])) {
    $delete_manager_id= $_REQUEST['delete_manager_id'];
//    echo "XXX: delete_manager_id= $delete_manager_id<br>";
    $role->removeRoleManagerID($delete_manager_id);
}



// get all infos to display in HTML
$role_title= $role->getTitle();
$role_key= $role->getRoleKey();
$provider=& $role->getRoleProvider();
$provider_id= $provider->getID();
$provider_attrs= $provider->getUserAttributes('commonName', 'privilegeName');
$managers_attrs= $role->getRoleAdministratorsAttributes('commonName');


// new providers: all portal and role admin (without actual provider)
$portal_and_role= array(USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR,USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR);
$new_providers_attrs= Administrator::getPrivilegedAdministratorsAttributes($portal_and_role, 'commonName');
foreach($new_providers_attrs as $idx => $new_provider_attrs) {
    if ($new_provider_attrs['user_id'] == $provider_id) {
        unset( $new_providers_attrs[$idx] ); // delete actual provider
        break;
    }
}

//
// new managers: all admin (without actual managers and provider)
$new_managers_attrs= Administrator::getPrivilegedAdministratorsAttributes(array(), 'commonName');
foreach($new_managers_attrs as $idx => $new_manager_attrs) {
    if ($new_manager_attrs['user_id'] == $provider_id) {
        unset( $new_managers_attrs[$idx] ); // delete actual provider
    }
    foreach($managers_attrs as $m_attrs) {
        if ($new_manager_attrs['user_id'] == $m_attrs['user_id']) {
            unset( $new_managers_attrs[$idx] ); // delete actual manager
            break;
        }
    }
}

//
// HTML content
//
include("include/adminHeader.inc"); 
?>

<!-- navigation menu -->
<p align="center">
  <span class="button-reference"><a href="edit-role.php?rid=<?= $rid ?>">Edit Role</a></span>
| <span class="button-reference"><a href="edit-role-aap.php?rid=<?= $rid ?>">Edit Standard Attributes</a></span>
| <span class="button-reference"><a href="edit-custom-role-aap.php?rid=<?= $rid ?>">Edit Custom Attributes</a></span>
</p>

<h1>Edit Role Administrators</h1>

<p>
You can change the role owner. Only Portal and Role Administrator privileged users are allowed to be role owner.
</p>

<!-- Role Provider -->
<form name="roleProviderFrom" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
<input type="hidden" name="rid" value="<?= $rid ?>" />
<input type="hidden" name="do_change_provider" value="1" />

<table width="90%" border="0" cellspacing="0" class="form-table">

<tr>
    <td colspan="2" class="form-head">
    Role Provider
    </td>
</tr>
<tr>
    <td width="30%" valign="top" class="form-label">
    Role Title (Key)
    </td>
    <td valign="top" class="form-entry">
    <?= $role_title ?> (<?= $role_key ?>)
    </td>
</tr>
<tr>
    <td width="30%" valign="top" class="form-label">
    Provider Name (Privilege)
    </td>
    <td valign="top" class="form-entry">
    <?= $provider_attrs['commonName'] ?> (<?= $provider_attrs['privilegeName'] ?>)
    </td>
</tr>
<?php if (!empty($new_providers_attrs)): ?>
<tr>
    <td width="30%">&nbsp;</td>
    <td class="form-control">
    <select name="new_provider_id">
    <?php foreach($new_providers_attrs as $new_p_attrs): ?>
        <option value="<?= $new_p_attrs['user_id'] ?>"><?= $new_p_attrs['commonName'] ?> (<?= $new_p_attrs['privilegeName'] ?>)</option>
    <? endforeach; ?>
    </select>
    <input type="submit" value="Change Owner">
    </td>
</tr>
<?php endif; ?>

</table>
</form>

<p>
You can manage the Role Administrators of this role. A Role Administrator will be able to manage the role regarding his Privilege.
</p>

<!-- Role Managers -->
<table width="90%" border="0" cellspacing="0" class="form-table">

<tr>
    <td colspan="3" class="form-head">
    Role Administrators
    </td>
</tr>
<tr>
    <td width="30%" valign="top" class="form-label">
    Role Title (Key)
    </td>
    <td colspan="2" valign="top" class="form-entry">
    <?= $role_title ?> (<?= $role_key ?>)
    </td>
</tr>

<?php
if (empty($managers_attrs)): ?>
<tr>
    <td valign="top" colspan="3" class="data-odd">
    No role administrator associated to this role.
    </td>
</tr>
<?php else:
    foreach($managers_attrs as $manager_attrs): ?>
        <tr>
        <td width="30%" valign="top" class="form-label">
        Administrator Name (Privilege)
        </td>
        <td width="50%" valign="top" class="form-entry">
        <?= $manager_attrs['commonName'] ?> (<?= $manager_attrs['privilegeName'] ?>)
        </td>
        
        <form name="removeManagerForm" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
        <input type="hidden" name="rid" value="<?= $rid ?>" />
        <input type="hidden" name="do_remove_manager" value="1" />
        <input type="hidden" name="delete_manager_id" value="<?= $manager_attrs['user_id'] ?>" />            
        <td width="20%" valign="top" class="form-entry">
        <input type="submit" value="Remove Administrator"/>
        </td>
        
        </form>
        </tr>
    <?php endforeach;
endif; ?>

<?php if (!empty($new_managers_attrs)): ?>
<tr>
    <form name="addManagerForm" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
    <input type="hidden" name="rid" value="<?= $rid ?>" />
    <input type="hidden" name="do_add_manager" value="1" />

    <td width="30%">&nbsp;</td>
    <td colspan="2" class="form-control">
    <select name="new_manager_id">
    <?php foreach($new_managers_attrs as $new_m_attrs): ?>
        <option value="<?= $new_m_attrs['user_id'] ?>"><?= $new_m_attrs['commonName'] ?> (<?= $new_m_attrs['privilegeName'] ?>)</option>
    <? endforeach; ?>
    </select>
    <input type="submit" value="Add Administrator"/>
    </td>
    </form>
</tr>
<?php endif; ?>
</table>



<?php
include("include/adminFooter.inc"); 
?>

