<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: list-orphan-users.php,v 1.13 2006/09/29 10:51:30 vlry Exp $
 */

/**
* This page displays a list of all orphan users and implements the business
* logic for deleting users from the database. 
*
*  This page can only be opened by administrator users with
* privilege Portal Administrator .
*
* @version $Revision: 1.13 $
* @package AdminPortal
*/

/**
 * Deletes a user from the database
 *
 * @param MySQLConnection $conn  an open MySQL connection
 * @param int $uid the user ID 
 *
 * @access private
 */
function delete_user($uid, $admin_id= 1) {
  $user=& User::reconstituteById($uid);
  if ($user) {
     $user->delete($admin_id);
  }
}


// bootstrap application
//
require_once("include/application.inc");

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("current user is not priviledged to enter this page"); 
}
Session::init($admin);

// check whether the current user is allowed to see details about
// this user
//
if (! $admin->canListUsers()) {
    ErrorHandling::fatalErrorAndExit("you do not have enough privileges to enter this page");
}


// process delete requests 
//
if (isset($_REQUEST["do_delete"])) {
    
    if ( $admin->canDeleteUser() ) {
        // delete the selected users
        foreach($_REQUEST as $key => $value) {
            if (substr($key, 0, 5) == "user-") {
                $id= substr($key, 5);
                if (is_numeric($id) && $id > 0) {
                    delete_user($id,$admin->getID());
                }
            }
        }
    }
} // end if  

//
// orderby in request and/or Session?
$orderby= 'surname'; // default sorting
if (Session::exists('user_order_by')) {
    $orderby= Session::get('user_order_by');
}
if (isset($_REQUEST['order_by'])) {
    $orderby= $_REQUEST['order_by'];
    Session::set('user_order_by', $orderby);
}
//XXX
//print "orderby: $orderby<br>";

// limit offset, count
$limit_offset= 0;
if (isset($_REQUEST['limit_offset'])) {
    $limit_offset= $_REQUEST['limit_offset'];
}
$limit_count= 100; // default 100 users/page
if (Session::exists('limit_count')) {
    $limit_count= Session::get('limit_count');
}
if (isset($_REQUEST['limit_count'])) {
    $limit_count= $_REQUEST['limit_count'];
    Session::set('limit_count', $limit_count);
}
//XXX
//print "limit_offset: $limit_offset<br>";
//print "limit_count: $limit_count<br>";

$n_users= User::getOrphansCount();
$n_pages= ceil( $n_users / $limit_count );
$page_n= ($limit_offset / $limit_count ) + 1;


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>
<script language="JavaScript">
<!--
function delete_user(id) {
    if (confirm("Do you really want to delete this user ?")) {
        select_user(id);
        document.form_user_list.submit();
    }
}

function num_selected_users() {
    var ret= 0;
    for(i=0; i<document.form_user_list.elements.length; i++) {
        var el= document.form_user_list.elements[i];
        if (el.name.substr(0,5) == "user-") {
            if (el.checked) {
                ret++; 
            }
        }
    }
    return ret;
}

function delete_selected_users() {
    if (num_selected_users() == 0) {
        alert("Please select at least one user first.");
    }
    else if (confirm("Do you really want to delete the selected users ?")) {
        document.form_user_list.submit();
    }
}

function select_user(id) {
    var user_id= "user-" + id;
    for(i=0; i<document.form_user_list.elements.length; i++) {
        var elt= document.form_user_list.elements[i];
        if (elt.name == user_id) {
            elt.checked= true; 
        }
    }
}

function select_all_users() {
    for(i=0; i<document.form_user_list.elements.length; i++) {
        var el = document.form_user_list.elements[i];
        if (el.name.substr(0,5) == "user-") {
            el.checked= true; 
        }
    }
}

function unselect_all_users() {
    for(i=0; i<document.form_user_list.elements.length; i++) {
        var el = document.form_user_list.elements[i];
        if (el.name.substr(0,5) == "user-") {
            el.checked= false;
        }
    }
}

function orderby(attr_name) {
    document.form_controller.order_by.value= attr_name;
    document.form_controller.submit();
}

function change_limit_count() {
    var lcount= parseInt(document.form_controller.limit_count_list.value);
    document.form_controller.limit_count.value= lcount;
    document.form_controller.limit_offset.value= 0;
    document.form_controller.submit();
}


function goto_page( page_n ) {
    var lcount= parseInt(document.form_controller.limit_count.value);
    document.form_controller.limit_offset.value= lcount * ( page_n - 1 );
    document.form_controller.submit();
}

function prev_page() {
    var loffset= parseInt(document.form_controller.limit_offset.value);
    var lcount= parseInt(document.form_controller.limit_count.value);
    document.form_controller.limit_offset.value= loffset - lcount;
    document.form_controller.submit();
}
function next_page() {
    var loffset= parseInt(document.form_controller.limit_offset.value);
    var lcount= parseInt(document.form_controller.limit_count.value);
    document.form_controller.limit_offset.value= loffset + lcount;
    document.form_controller.submit();
}
// -->
</script>

<h1>Users without any subscription (Orphans)</h1>
  
 <form name="form_controller" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="order_by" value="<?= $orderby ?>" />
  <input type="hidden" name="limit_offset" value="<?= $limit_offset ?>" />
  <input type="hidden" name="limit_count" value="<?= $limit_count ?>" />

<?= $n_users ?> orphan users found (
<select name="limit_count_list" onChange="change_limit_count()">
<?php
    // drop down list 10, 25, 50, 100, 200
    $limit_counts= array( 10, 25, 50, 100, 200 );
    foreach( $limit_counts as $count ) {
        $selected= "";
        if ($count == $limit_count) $selected= "selected ";
        print "<option value=\"$count\" $selected>$count";
    }
?>
</select>
users/page on <?= $n_pages ?> pages)<br>

</form>

 <?php
 if ($n_pages > 1) {
    // show << | 1 | 2 | ... | n | >> control
    if ($page_n > 1) { 
        print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
    }
    else {
        print "&lt;&lt; ";
    }
    if ($n_pages != 0) {
        foreach( range(1,$n_pages) as $page ) {
            if ($page == $page_n) {
                print "| <b>$page</b> ";
            }
            else {
                print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
            }
        }
    }
    if ($page_n < $n_pages ) {
        print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
    }
    else {
        print "| &gt;&gt;";
    }
 }
 ?>
 <p/>
  
<form name="form_user_list" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="do_delete" value="1" />
  <input type="hidden" name="order_by" value="<?= $orderby ?>" />
  <input type="hidden" name="limit_offset" value="<?= $limit_offset ?>" />
  <input type="hidden" name="limit_count" value="<?= $limit_count ?>" />

  <table border="0" cellspacing="0" class="data-table">

    <tr>
      <th class="data" valign="top" width="1%">&nbsp;</th>
      <th class="data" valign="top"><a href="javascript:orderby('user_uid')" title="Sort by User ID">
      <?php if ($orderby == 'user_uid') print "<i>User ID</i>";
            else print "User ID"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('surname')" title="Sort by Lastname">
      <?php if ($orderby == 'surname') print "<i>Lastname</i>";
            else print "Lastname"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('givenName')" title="Sort by Firstname">
      <?php if ($orderby == 'givenName') print "<i>Firstname</i>";
            else print "Firstname"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('privilegeName')" title="Sort by Privilege">
      <?php if ($orderby == 'privilegeName') print "<i>Privilege</i>";
            else print "Privilege"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('user_last_login')" title="Sort by Login Date">
      <?php if ($orderby == 'user_last_login') print "<i>Last Login</i>";
            else print "Last Login"; ?>
      </a></th>
      <th class="data last" >Actions</th>
    </tr>  
    
    <?php
    $users_attrs=& User::getAllOrphansAttributes($orderby, $limit_offset, $limit_count);
    $i= 0;
    foreach ($users_attrs as $user_attrs):
       $style= ($i++ % 2 == 0) ? "data-even" : "data-odd";
       $user_id= $user_attrs[ATTR_USER_ID];
       $uid= $user_attrs[ATTR_USER_UID];
       $privilegename= $user_attrs[ATTR_PRIVILEGENAME];
       $firstname= $user_attrs[ATTR_GIVENNAME];
       $lastname= $user_attrs[ATTR_SURNAME];
       if (empty($firstname)) $firstname= '&nbsp;';
       if (empty($lastname)) $lastname= '&nbsp;';
       $lastlogin= $user_attrs[ATTR_LAST_LOGIN];
       ?>
      <tr>
        <!-- Checkbox -->
        <td valign="top" class="<?= $style ?>">
         <input type="checkbox" name="user-<?= $user_id ?>" value="" />
        </td>
      
        <!-- User ID -->
        <td valign="top" class="<?= $style ?>">
        <?= $uid ?> 
        </td>
                
        <!-- Lastname -->
        <td valign="top" class="<?= $style ?>">
        <?= $lastname ?>
        </td>

        <!-- Firstname -->
        <td valign="top" class="<?= $style ?>">
        <?= $firstname ?>
        </td>

        <!-- Privilege Name -->
        <td valign="top" class="<?= $style ?>">
        <?= $privilegename ?>
        </td>
        
        <!-- Last Login -->
        <td valign="top" class="<?= $style ?>">
        <?= Formatting::formatMysqlDate($lastlogin,true) ?>
        </td>
  
        <td valign="top" class="<?= $style ?> last">
          <a href="edit-user.php?uid=<?= $user_id ?>">Edit</a>
          | <a href="javascript:delete_user(<?= $user_id ?>);">Delete</a>
        </td>
      </tr>

    <?php endforeach; ?>
    
    <tr>
      <td class="data-even"><?php echo count($users_attrs) ?> </td>
      <td colspan="7" class="data-even last">
        <a href="javascript:select_all_users()">Select All</a> 
        | <a href="javascript:unselect_all_users()">Unselect All</a>
      </td>
    </tr>
    
    <tr>
      <td colspan="7" class="form-control">
        <input type="button" value="Delete Selected" onClick="delete_selected_users()" />
      </td>
    </tr>    
  </table>
  </form>
<p>
 <?php
 if ($n_pages > 1) {
    // show << | 1 | 2 | ... | n | >> control
    if ($page_n > 1) { 
        print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
    }
    else {
        print "&lt;&lt; ";
    }
    foreach( range(1,$n_pages) as $page ) {
        if ($page == $page_n) {
            print "| <b>$page</b> ";
        }
        else {
            print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
        }
    }
    if ($page_n < $n_pages ) {
        print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
    }
    else {
        print "| &gt;&gt;";
    }
 }
 ?>
 <p/>
<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
