<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: list-subscriptions.php,v 1.85 2006/09/28 12:51:38 vlry Exp $
 */

/**
* Displays two list: a list of all processed (accepted or 
* rejected) subscriptions and a list of pending subscription
* rejects per role.   
* 
* @version $Revision: 1.85 $
* @package AdminPortal
*/

/**
 * Ensures the request parameters are valid.
 *
 * @access private
 *
 */
function ensure_request_parameters_valid() {
  if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing");
  }
  $rid = $_REQUEST["rid"];
  if (! is_numeric($rid) || $rid < 0) {
    ErrorHandling::fatalErrorAndExit("illegal value '$rid' for request parameter 'rid'. Aborting.");
  }
}


/**
 * Filters the mobile telephone number. 
 *
 * Replace first '+' with '00' and then suppress all 
 * non-digit characters.
 *
 * @param string $mobile The mobile number.
 * @return string The filtered mobile number.
 */
function filter_mobile($mobile) {
    $mobile= trim($mobile);
    $mobile= str_replace('+', '00', $mobile);
    $mobile= ereg_replace("[^[:digit:]]", "", $mobile);
    return $mobile;
}


/**
 * Request handler for request 'do_accept'.  Processes request parameters 
 * in the global array $_REQUEST
 *
 * @param MySQLConnection $conn         a MySQL connection
 * @param Administrator  $admin        an admin user (an instance of Administrator)
 *
 * @access private
 */
function handle_do_accept($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }  
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }       
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            
            // accept subscription
            $subscription->accept($admin->getID());

            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_ACCEPTED, false, $data);
            }   
        }
    }
}

/**
 * Request handler for request 'do_delete'.  
 * Processes request parameters in the global array $_REQUEST
 *
 * @param Administrator  $admin        an admin user (an instance of Administrator)
 *
 * @access private
 */
function handle_do_delete($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }       
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }       
            // delete subscription
            $role=& Role::reconstituteById($rid);
            Subscription::delete($user,$role,$admin->getID());
            //$user->unsubscribeFromRole($rid);

            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_DELETED, false, $data);
            }   
        }
    }
}


/**
 * Request handler for request 'do_reject'. Processes request parameters in 
 * the global array $_REQUEST
 *
 * @param Administrator $admin        an admin user (an instance of Administrator)
 *
 * 
 * @access private
 */
function handle_do_reject($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }       
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            
            // reject subscription
            $subscription->reject($admin->getID());
            
            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_REJECTED, false, $data);
            }   
        }
    }
}

/**
 * Request handler for request 'do_suspend'. Processes request parameters in
 * the global array $_REQUEST
 *
 * @param Administrator $admin        an admin user (an instance of Administrator)
 *
 * @access private
 *
 */
function handle_do_suspend($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            
            // suspend subscription
            $subscription->suspend($admin->getID());
            
            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_SUSPENDED, false, $data);
            }   
        }
    }
}


//
// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("current user is not priviledged to enter this page"); 
}
Session::init($admin);

// make sure 'rid' identifies an existing role
//
$rid = $_REQUEST["rid"];
$role =& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("there is no role with id '$rid'");
}

// make sure the current administrator can manage this
// role
//
if (! $admin->canManageSubscriptions($role)) {
  ErrorHandling::fatalErrorAndExit("you do not have enough privileges to manage subscriptions of role '$rid'");
}

//
// process requests action=
//
if (isset($_REQUEST['action'])) {
    if ($_REQUEST['action'] == "do_accept" || $_REQUEST['action'] == "do_resume") {
       handle_do_accept($admin);
    } elseif ($_REQUEST['action'] == "do_reject") {
       handle_do_reject($admin);
    } elseif ($_REQUEST['action'] == "do_suspend") {
       handle_do_suspend($admin);
    } elseif ($_REQUEST['action'] == "do_delete") {
       handle_do_delete($admin);
    }
}

//
// sorting
//
$orderby= 'commonName'; // default
if (Session::exists('subscription_order_by')) {
    $orderby= Session::get('subscription_order_by');
}
if (isset($_REQUEST['order_by'])) {
    $orderby= $_REQUEST['order_by'];
    Session::set('subscription_order_by', $orderby);
}
//XXX
//print "orderby: $orderby<br>";


//
// limit offset, count. 
// only for confirmed subscriptions
//
$limit_offset= 0;
if (isset($_REQUEST['limit_offset'])) {
    $limit_offset= $_REQUEST['limit_offset'];
}
$limit_count= 100; // default 100 subs/page
if (Session::exists('limit_count')) {
    $limit_count= Session::get('limit_count');
}
if (isset($_REQUEST['limit_count'])) {
    $limit_count= $_REQUEST['limit_count'];
    Session::set('limit_count', $limit_count);
}
//XXX print "limit_offset: $limit_offset<br>"; print "limit_count: $limit_count<br>";

// subs counts
$n_pending_subscriptions= $role->getSubscriptionsCount(SUBSCRIPTION_STATE_PENDING);
$n_subscriptions= $role->getSubscriptionsCount(SUBSCRIPTION_STATE_ACCEPTED,
                                                   SUBSCRIPTION_STATE_REJECTED,
                                                   SUBSCRIPTION_STATE_SUSPENDED);

$n_pages= ceil( $n_subscriptions / $limit_count );
$page_n= ($limit_offset / $limit_count ) + 1;


/* ************************************************* */
/*               HTML content                        */
/* ************************************************* */
include("include/adminHeader.inc");
?>

<script language="JavaScript">
<!--
function delete_subscription(formname,sid) {
  if (confirm("Are you sure you want to delete this subscription?")) {
    select_subscription(formname,sid);
    document[formname].action.value = "do_delete"; 
    document[formname].submit();
  }
}

function accept_subscription(formname,sid) {
    select_subscription(formname,sid);
    accept_selected(formname);
}

function suspend_subscription(formname,sid) {
    select_subscription(formname,sid);
    suspend_selected(formname);
}
function resume_subscription(formname,sid) {
    select_subscription(formname,sid);
    resume_selected(formname);
}

function reject_subscription(formname,sid) {
    select_subscription(formname,sid);
    reject_selected(formname);
}

function num_selected(formname) {
  var ret = 0;
  for(i=0; i < document[formname].elements.length;i++) {
    var el = document[formname].elements[i];
    if (el.name.indexOf("sub-") == 0) {
      if (el.checked) ret++; 
    }
  }
  return ret;
}

function accept_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  document[formname].action.value = "do_accept";  
  document[formname].submit();
}

function resume_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  document[formname].action.value = "do_resume";  
  document[formname].submit();
}

function suspend_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  document[formname].action.value = "do_suspend";  
  document[formname].submit();
}

function reject_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  document[formname].action.value = "do_reject";  
  document[formname].submit();
}

function delete_selected(formname) {
  var n_subs= num_selected(formname);
  if (n_subs == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  if (confirm("Are you sure you want to delete the selected subscriptions?")) {
    if (confirm("Really sure? Delete " + n_subs + " subscriptions?")) {
        document[formname].action.value = "do_delete";  
        document[formname].submit();
    }
  }
}

function select_subscription(formname,id) {
  var sub_id= "sub-" + id;
  document[formname][sub_id].checked= true;
}

function select_all(formname) {
  for(i=0; i < document[formname].elements.length;i++) {
    var el = document[formname].elements[i];
    if (el.name.indexOf("sub-") == 0) {
      el.checked= true; 
    }
  }
}

function unselect_all(formname) {
  for(i=0; i < document[formname].elements.length;i++) {
    var el = document[formname].elements[i];
    if (el.name.indexOf("sub-") == 0) {
      el.checked =  false;
    }
  }
}

function mail_to_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  recs = "";
  for(i=0; i < document[formname].elements.length; i++) {
    e = document[formname].elements[i];
    if (e.name.indexOf("mail-") == 0) {
       recs += (recs == "") ? "" : ", ";
       recs += e.value;
    }
  }
  //alert("mailto:" + recs);
  window.open('email.php?recs=' + recs, 'email', 'scrollbars=no, toolbar=no, location=no, menubar=no, width=600, height=360');
}

function sms_to_selected(formname) {
  if (num_selected(formname) == 0) {
    alert("Please select at least one subscription first.")
    return;
  }
  recs = "";
  for(i=0; i < document[formname].elements.length; i++) {
    e = document[formname].elements[i];
    if (e.name.substr(0,4) == "sms-") {
       recs += (recs == "") ? "" : ",";
       recs += e.value;
    }
  }
  //XXX: alert("sms_to_all:" + recs);
  window.open('email.php?recs=' + recs + '&sms=1', 'email', 'scrollbars=no, toolbar=no, location=no, menubar=no, width=600, height=300');
}

function mail_to(email) {
  window.open('email.php?recs=' + email, 'email', 'scrollbars=no, toolbar=no, location=no, menubar=no, width=600, height=330');
}

function sms_to(mobile) {
    rcpt= mobile;
    //XXX: alert("sms_to:" + rcpt); 
  window.open('email.php?recs=' + rcpt + '&sms=1', 'email', 'scrollbars=no, toolbar=no, location=no, menubar=no, width=600, height=300');
}

function orderby(forname,attr_name) {
    document[forname].order_by.value= attr_name;
    document[forname].submit();
}

function change_limit_count() {
    var lcount= parseInt(document.form_subs.limit_count_list.value);
    document.form_subs.limit_count.value= lcount;
    document.form_subs.limit_offset.value= 0;
    document.form_subs.submit();
}


function goto_page( page_n ) {
    var lcount= parseInt(document.form_subs.limit_count.value);
    document.form_subs.limit_offset.value= lcount * ( page_n - 1 );
    document.form_subs.submit();
}

function prev_page() {
    var loffset= parseInt(document.form_subs.limit_offset.value);
    var lcount= parseInt(document.form_subs.limit_count.value);
    document.form_subs.limit_offset.value= loffset - lcount;
    document.form_subs.submit();
}
function next_page() {
    var loffset= parseInt(document.form_subs.limit_offset.value);
    var lcount= parseInt(document.form_subs.limit_count.value);
    document.form_subs.limit_offset.value= loffset + lcount;
    document.form_subs.submit();
}
// -->
</script>

<h1>Subscriptions for Role: <?= $role->getTitle() ?> [<?= $role->getRoleKey() ?>]</h1>


<?php
 if ($n_pending_subscriptions > 0):
    // if any, display pending subscriptions
    $pending_state= array(SUBSCRIPTION_STATE_PENDING);
    $pending_subs_attrs= $role->getSubscriptionsAttributes($pending_state, $orderby);
?>
     
  <h3>Pending Subscriptions</h3>

  <!-- **************************************************** -->
  <!--        List of Pending Subscriptions                 -->
  <!-- **************************************************** -->
  
 
  <form name="form_pending_subs" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="rid" value="<?= $rid ?>" />
  <input type="hidden" name="action" value="unknown" />
  <input type="hidden" name="order_by" value="<?= $orderby ?>" />
  <input type="hidden" name="limit_offset" value="<?= $limit_offset ?>" />
  <input type="hidden" name="limit_count" value="<?= $limit_count ?>" />
  
  <table width="95%" border="0" cellspacing="0" class="data-table">

    <!-- HEADERS pending subs -->
    <tr>
      <th class="data" width="1%">&nbsp;</th>
      <th class="data">
      <a href="javascript:orderby('form_pending_subs','commonName')" title="Sort by User Name">
      <?php if ($orderby == 'commonName') print "<i>User Name</i>";
            else print "User Name"; ?>
      </a>
      (<a href="javascript:orderby('form_pending_subs','user_uid')"  title="Sort by User ID">
      <?php if ($orderby == 'user_uid') print "<i>User ID</i>";
            else print "User ID"; ?>
      </a>)
      </th>
      <th class="data">
      <a href="javascript:orderby('form_pending_subs','subscription_state')" title="Sort by Status">
      <?php if ($orderby == 'subscription_state') print "<i>Status</i>";
            else print "Status"; ?>
      </a>
      </th>
      <th class="data">
      <a href="javascript:orderby('form_pending_subs','date_created')" title="Sort by Date">
      <?php if ($orderby == 'date_created') print "<i>Subscribed On</i>";
            else print "Subscribed On"; ?>
      </a>
      </th>
      <th class="data" >Actions</th>
    </tr>  
    
    <?php $i= 0; 
    foreach ($pending_subs_attrs as $sub_attrs):
       $style = ($i++ % 2 == 0) ? "data-even" : "data-odd";
       $user_id= $sub_attrs['subscription_user'];
       $user_uid= $sub_attrs['user_uid'];
       $user_commonName= $sub_attrs['commonName'];
       $user_mail= null;
       if (isset($sub_attrs['mail'])) {
           $user_mail= $sub_attrs['mail'];
       }
       $user_sms= null;
       if (isset($sub_attrs['mobileTelephonNumber'])) {
           $user_sms= $sub_attrs['mobileTelephonNumber'];
       }
       // subscription identifier
       $sub_id= "$user_id-$rid";
       $sub_date_created= $sub_attrs['date_created'];
       ?>
    
    <tr>
      <td valign="top" class="<?= $style ?>">
         <input type="checkbox" name="sub-<?= $sub_id ?>" />
      </td>
        
      <td valign="top" class="<?= $style ?>">
         <?= $user_commonName ?> (<?= $user_uid ?>)
      </td>
      <td valign="top" class="<?= $style ?>">
      Pending
      </td>
      <td valign="top" class="<?= $style ?>">
         <?= Formatting::formatMySQLDate( $sub_date_created ) ?>
      </td>
      <td valign="top" class="<?= $style ?> last">
        <a href="show-subscription-details.php?uid=<?= $user_id ?>&rid=<?= $rid ?>" title="View Subscription Details">Details</a>
      | <a href="javascript:accept_subscription('form_pending_subs','<?= $sub_id ?>')" title="Accept Subscription">Accept</a> 
<!--      | <a href="javascript:reject_subscription('form_pending_subs','<?= $sub_id ?>')">Reject</a> -->
      | <a href="javascript:delete_subscription('form_pending_subs','<?= $sub_id ?>')" title="Delete Subscription">Delete</a> 
       
      <?php if (!empty($user_mail)): ?>
      | <a href="new-invitation.php?rid=<?= $rid ?>&to=<?= $user_id ?>" title="Send an Invitation">Invite</a> 
      | <a href="javascript:mail_to('<?= $user_mail ?>')" title="Send an Email to User">Send Email</a> 
      <input type="hidden" name="mail-<?= $user_id ?>" value="<?= $user_mail ?>" />
      <?php  endif; ?>

      <?php if (!empty($user_sms)): 
            $user_mobile= filter_mobile( $user_sms ); ?>
      | <a href="javascript:sms_to('<?= $user_mobile ?>')" title="Send a SMS to User">Send SMS</a> 
      <input type="hidden" name="sms-<?= $user_id ?>" value="<?= $user_mobile ?>" />
      <?php endif; ?>
      
      </td>
      
    </tr>
    
    <?php endforeach; ?>


    <tr>
      <td valign="top" class="data-odd" colspan="2">
        <a href="javascript:select_all('form_pending_subs')">Select All</a> 
      | <a href="javascript:unselect_all('form_pending_subs')">Unselect All</a><br/>
      </td>
      <td valign="top" class="data-odd last" colspan="3">
        &nbsp;
      </td>
    </tr>  

    <tr>
      <td valign="top" class="form-control" align="left" colspan="5">
        <input type="button" name="do_accept" 
           value="Accept Selected" onClick="javascript:accept_selected('form_pending_subs')">        
<!--        <input type="button" name="do_reject" 
          value="Reject Selected" onClick="javascript:reject_selected('form_pending_subs')"> -->
        <input type="button" name="do_send_email" 
          value="Mail to Selected" onClick="javascript:mail_to_selected('form_pending_subs')">        
        <input type="button" name="do_send_sms" 
          value="SMS to Selected" onClick="javascript:sms_to_selected('form_pending_subs')">        

      </td>
    </tr>  

  </table>
  </form>
  <?php endif; // not empty ?>  
  
  
  
  <!-- **************************************************** -->
  <!--        List of Confirmed Subscriptions    -->
  <!-- **************************************************** -->
  

  <h3>Confirmed Subscriptions</h3>
  
  <form name="form_subs" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="rid" value="<?= $rid ?>" />
  <input type="hidden" name="action" value="unknown" />
  <input type="hidden" name="order_by" value="<?= $orderby ?>" />
  <input type="hidden" name="limit_offset" value="<?= $limit_offset ?>" />
  <input type="hidden" name="limit_count" value="<?= $limit_count ?>" />

<?= $n_subscriptions ?> subscriptions (<select name="limit_count_list" onChange="change_limit_count()">
<?php
    // drop down list 10, 25, 50, 100, 200
    $limit_counts= array( 10, 25, 50, 100, 200 );
    foreach( $limit_counts as $count ) {
        $selected= "";
        if ($count == $limit_count) $selected= "selected ";
        print "<option value=\"$count\" $selected>$count";
    }
?>
</select> subs/page on <?= $n_pages ?> pages)<br>
 <?php
    if ($n_pages > 1) {
        // show << | 1 | 2 | ... | n | >> control
        if ($page_n > 1) { 
            print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
        }
        else {
            print "&lt;&lt; ";
        }
        foreach( range(1,$n_pages) as $page ) {
            if ($page == $page_n) {
                print "| <b>$page</b> ";
            }
            else {
                print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
            }
        }
        if ($page_n < $n_pages ) {
            print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
        }
        else {
            print "| &gt;&gt;";
        }
    }
 ?>
 <p>
  <table width="95%" border="0" cellspacing="0" class="data-table">

    <!-- HEADERS subs -->
    <tr>
      <th class="data" width="1%">&nbsp;</th>
      <th class="data">
      <a href="javascript:orderby('form_subs','commonName')" title="Sort by User Name">
      <?php if ($orderby == 'commonName') print "<i>User Name</i>";
            else print "User Name"; ?>
      </a>
      (<a href="javascript:orderby('form_subs','user_uid')"  title="Sort by User ID">
      <?php if ($orderby == 'user_uid') print "<i>User ID</i>";
            else print "User ID"; ?>
      </a>)
      </th>
      <th class="data">
      <a href="javascript:orderby('form_subs','subscription_state')" title="Sort by Status">
      <?php if ($orderby == 'subscription_state') print "<i>Status</i>";
            else print "Status"; ?>
      </a>
      </th>
      <th class="data">
      <a href="javascript:orderby('form_subs','date_created')" title="Sort by Date">
      <?php if ($orderby == 'date_created') print "<i>Subscribed On</i>";
            else print "Subscribed On"; ?>
      </a>
      </th>
      <th class="data last">Actions</th>
    </tr>  

    
  <?php  
    if ($n_subscriptions < 1): 
  ?>
    
    <tr>
      <td width="100%" valign="top" class="data-odd last" colspan="5">
         Currently, there are no accepted, rejected or suspended subscriptions for this role.
      </td>
    
  <?php 
     else: 
  ?>
  
    <?php
        // display all not pending subscriptions
        $not_pending_states= array(SUBSCRIPTION_STATE_ACCEPTED,
                                   SUBSCRIPTION_STATE_REJECTED,
                                   SUBSCRIPTION_STATE_SUSPENDED);
        $subs_attrs=& $role->getSubscriptionsAttributes($not_pending_states,$orderby,$limit_offset,$limit_count);
        $i= 0;
        foreach ($subs_attrs as $sub_attrs):
            $style = ($i++ % 2 == 0) ? "data-even" : "data-odd";
            $user_id= $sub_attrs['subscription_user'];
            $user_uid= $sub_attrs['user_uid'];
            $user_commonName= $sub_attrs['commonName'];
            $user_mail= null;
            if (isset($sub_attrs['mail'])) {
                $user_mail= $sub_attrs['mail'];
            }
            $user_sms= null;
            if (isset($sub_attrs['mobileTelephonNumber'])) {
                $user_sms= $sub_attrs['mobileTelephonNumber'];
            }
            // subscription identifier
            $sub_id= "$user_id-$rid";
            $sub_date_created= $sub_attrs['date_created'];
            $sub_state= $sub_attrs['subscription_state'];
    ?>
    
      <tr>
        <td valign="top" class="<?= $style ?>">
           <input type="checkbox" name="sub-<?= $sub_id ?>" />         
        </td>
          
        <td valign="top" class="<?= $style ?>">
         <?= $user_commonName ?> (<?= $user_uid ?>)
        </td>
        
        <td valign="top" class="<?= $style ?>">
        <?php          
        switch($sub_state) {
        case  SUBSCRIPTION_STATE_ACCEPTED:
            echo "Accepted"; break;
        case  SUBSCRIPTION_STATE_REJECTED:
            echo "Rejected"; break;
        case  SUBSCRIPTION_STATE_SUSPENDED:          
            echo "Suspended"; break;
        case SUBSCRIPTION_STATE_PENDING:
            echo "Pending"; break;        
        default:
            echo "Unknown ($sub_status)"; break;                  
        } 
        ?>
        </td>
        
        <td valign="top" class="<?= $style ?>">
           <?= Formatting::formatMySQLDate( $sub_date_created ) ?>
        </td>
        
        <!-- ACTIONS -->
        <td valign="top" class="<?= $style ?> last">
        <a href="show-subscription-details.php?uid=<?= $user_id ?>&rid=<?= $rid ?>" title="View Subscription Details">Details</a>
        | <a href="javascript:delete_subscription('form_subs','<?= $sub_id ?>')" title="Delete Subscription">Delete</a>
  
        <?php if ($sub_state == SUBSCRIPTION_STATE_ACCEPTED): ?>
<!--        | <a href="javascript:reject_subscription('form_subs','<?= $sub_id ?>')">Reject</a> -->
        | <a href="javascript:suspend_subscription('form_subs','<?= $sub_id ?>')" title="Suspend Subscription">Suspend</a>
        <?php  elseif ($sub_state == SUBSCRIPTION_STATE_REJECTED): ?>
        | <a href="javascript:accept_subscription('form_subs','<?= $sub_id ?>')" title="Accept Subscription">Accept</a>
        <?php  elseif ($sub_state == SUBSCRIPTION_STATE_SUSPENDED): ?>
        | <a href="javascript:resume_subscription('form_subs','<?= $sub_id ?>')" title="Resume Subscription">Resume</a>
<!--        | <a href="javascript:reject_subscription('form_subs','<?= $sub_id ?>')">Reject</a> -->
        <?php  endif; ?>
  
        <?php  if (!empty($user_mail)): ?>
            <?php  if ($sub_state == SUBSCRIPTION_STATE_REJECTED or $sub_state == SUBSCRIPTION_STATE_SUSPENDED): ?>
        | <a href="new-invitation.php?rid=<?= $rid ?>&to=<?= $user_id ?>" title="Send an Invitation to User">Invite</a> 
            <?php endif; ?>
        | <a href="javascript:mail_to('<?= $user_mail ?>')" title="Send an Email to User">Send Email</a> 
        <input type="hidden" name="mail-<?= $user_id ?>" value="<?= $user_mail ?>" />
        <?php  endif; ?>
  
        <?php  
            if (!empty($user_sms)): 
                $user_mobile= filter_mobile($user_sms); ?>
        | <a href="javascript:sms_to('<?= $user_mobile ?>')" title="Send a SMS to User">Send SMS</a> 
        <input type="hidden" name="sms-<?= $user_id ?>" value="<?= $user_mobile ?>" />
        <?php  endif; ?>
  
        </td>
        
      </tr>
    
    <?php endforeach; ?>


    <tr>
      <td valign="top" class="data-odd" colspan="2">
        <a href="javascript:select_all('form_subs')">Select All </a> 
      | <a href="javascript:unselect_all('form_subs')">Unselect All</a><br/>
      </td>
      <td valign="top" class="data-odd last" colspan="3">
        &nbsp;
      </td>
    </tr>  

    <tr>
      <td valign="top" class="form-control" align="left" colspan="5">
        <input type="button" name="do_accept" value="Accept Selected" onClick="javascript:accept_selected('form_subs')"> 
<!--        <input type="button" name="do_reject" value="Reject Selected" onClick="javascript:reject_selected('form_subs')"> -->
        <input type="button" name="do_delete" value="Delete Selected" onClick="javascript:delete_selected('form_subs')"> 
        <input type="button" name="do_suspend" value="Suspend Selected" onClick="javascript:suspend_selected('form_subs')"> 
        <input type="button" name="do_send_mail" value="Mail to Selected" onClick="javascript:mail_to_selected('form_subs')"> 
        <input type="button" name="do_send_sms" value="SMS to Selected" onClick="javascript:sms_to_selected('form_subs')"> 
      </td>
    </tr>  
    
  <?php 
    endif; // if n_subs < 1 else
  ?>
    
  </table>
  </form>

<p>
 <?php
    if ($n_pages > 1) {
        // show << | 1 | 2 | ... | n | >> control
        if ($page_n > 1) { 
            print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
        }
        else {
            print "&lt;&lt; ";
        }
        foreach( range(1,$n_pages) as $page ) {
            if ($page == $page_n) {
                print "| <b>$page</b> ";
            }
            else {
                print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
            }
        }
        if ($page_n < $n_pages ) {
            print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
        }
        else {
            print "| &gt;&gt;";
        }
    }
 ?>    

<p>

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>

