<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: list-user-subscriptions.php,v 1.21 2006/09/26 13:35:46 vlry Exp $
 */


/**
* This page displays a list of all subscription for the given user.
* 
* @version $Revision: 1.21 $
* @package AdminPortal
*/


/**
 * Accepts the selected subscriptions.
 * @access private
 */
function handle_do_accept($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }  
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }       
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            
            // accept subscription
            $subscription->accept($admin->getID());

            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_ACCEPTED, false, $data);
            }   
        }
    }
}

/**
 * Deletes the selected subscriptions
 * @access private
 */
function handle_do_delete($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }       
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }       
            // delete subscription
            $role=& Role::reconstituteById($rid);
            Subscription::delete($user,$role,$admin->getID());
            //$user->unsubscribeFromRole($rid);

            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_DELETED, false, $data);
            }   
        }
    }
}

/**
 * Suspends the selected subscriptions.
 * @access private
 */
function handle_do_suspend($admin) {
    foreach ($_REQUEST as $key => $value) {
        if (ereg("^sub-([0-9]+)-([0-9]+)$", $key, $regs)) {
            $uid = $regs[1];
            $rid = $regs[2];
            $subscription=& Subscription::reconstituteByIDs($uid,$rid);
            if (is_null($subscription)) {
                continue;
            }
            $user=& $subscription->getUser();
            if (is_null($user)) {
                continue;
            }
            if (! $admin->canModifySubscription($subscription)) {
                continue;
            }
            
            // suspend subscription
            $subscription->suspend($admin->getID());
            
            // send out notification message
            $admin_attrs= $admin->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
            $message = new Messaging();
            if ($message->is_enabled() && 
                !empty($admin_attrs[ATTR_EMAIL]) && !empty($user_attrs[ATTR_EMAIL])) {
               $from= $admin_attrs[ATTR_EMAIL];
               $to= $user_attrs[ATTR_EMAIL];
               $data= array();
               $data['from_name']= $admin_attrs[ATTR_COMMONNAME];
               $data['to_name']= $user_attrs[ATTR_COMMONNAME];
               $res= $subscription->getRole();
               $data['role_name']= $res->getTitle();
               $message->stateChanged($from, $to, SUBSCRIPTION_SUSPENDED, false, $data);
            }   
        }
    }
}


//
// bootstrap application
//
require_once("include/application.inc");

// check require request param
if (! isset($_REQUEST["uid"])) {
    ErrorHandling::fatalErrorAndExit("Request parameter 'uid' is missing");
}

// resume the current session and make sure we have a valid session
Session::ensureAdminSessionValid();
$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);


// recreate user
$user_id= $_REQUEST["uid"];
$user=& User::reconstituteById($user_id);
if (is_null($user)) {
  ErrorHandling::fatalErrorAndExit("No user with such id ($user_id).");
}

// check access right
//if (! $admin->canEditUser($user)) {
//  ErrorHandling::fatalErrorAndExit("Not have enough privileges to list subscription of this user.");
//}

//
// process requests action=
//
if (isset($_REQUEST['action'])) {
    if ($_REQUEST['action'] == "do_accept" || $_REQUEST['action'] == "do_resume") {
       handle_do_accept($admin);
    } elseif ($_REQUEST['action'] == "do_suspend") {
       handle_do_suspend($admin);
    } elseif ($_REQUEST['action'] == "do_delete") {
       handle_do_delete($admin);
    }
}


// sorting
$orderby= 'role_title'; // default
if (Session::exists('usersubs_order_by')) {
    $orderby= Session::get('usersubs_order_by');
}
if (isset($_REQUEST['order_by'])) {
    $orderby= $_REQUEST['order_by'];
    Session::set('usersubs_order_by', $orderby);
}
//XXX print "orderby: $orderby<br>";

// limit offset, count
$limit_offset= 0;
if (isset($_REQUEST['limit_offset'])) {
    $limit_offset= $_REQUEST['limit_offset'];
}
$limit_count= 100; // default 100 subs/page
if (Session::exists('limit_count')) {
    $limit_count= Session::get('limit_count');
}
if (isset($_REQUEST['limit_count'])) {
    $limit_count= $_REQUEST['limit_count'];
    Session::set('limit_count', $limit_count);
}
//XXX
//print "limit_offset: $limit_offset<br>";
//print "limit_count: $limit_count<br>";

$n_subscriptions= $user->getSubscriptionsCount();
$n_pages= ceil( $n_subscriptions / $limit_count );
$page_n= ($limit_offset / $limit_count ) + 1;

/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

<script language="JavaScript">
<!--
function num_selected_subscriptions() {
  var ret = 0;
  for(i=0; i < document.form_subs_list.elements.length;i++) {
    var el = document.form_subs_list.elements[i];
    if (el.name.indexOf("sub-") == 0) {
      if (el.checked) ret++; 
    }
  }
  return ret;
}

function delete_subscription(sid) {
  if (confirm("Are you sure you want to delete this subscription?")) {
    select_subscription(sid);
    document.form_subs_list.action.value= "do_delete";
    document.form_subs_list.submit();
  }
}

function delete_selected() {
    var n_subs= num_selected_subscriptions();
    if (n_subs == 0) {
        alert("Please select at least one subscription first.")
        return;
    }
    if (confirm("Are you sure you want to delete these subscriptions?")) {
        if (confirm("Really sure? Delete " + n_subs + "  subscriptions?")) {
            document.form_subs_list.action.value= "do_delete";
            document.form_subs_list.submit();
        }
    }
}

function accept_subscription(sid) {
    select_subscription(sid);
    accept_selected();
}

function accept_selected() {
    if (num_selected_subscriptions() == 0) {
        alert("Please select at least one subscription first.")
        return;
    }
    document.form_subs_list.action.value= "do_accept";
    document.form_subs_list.submit();
}

function suspend_subscription(sid) {
    select_subscription(sid);
    suspend_selected();
}

function suspend_selected() {
    if (num_selected_subscriptions() == 0) {
        alert("Please select at least one subscription first.")
        return;
    }
    document.form_subs_list.action.value= "do_suspend";
    document.form_subs_list.submit();
}

function resume_subscription(sid) {
    select_subscription(sid);
    resume_selected();
}

function resume_selected() {
    if (num_selected_subscriptions() == 0) {
        alert("Please select at least one subscription first.")
        return;
    }
    document.form_subs_list.action.value= "do_resume";
    document.form_subs_list.submit();
}

function select_subscription(sid) {
    var sub_id= "sub-" + sid;
    document.form_subs_list[sub_id].checked= true; 
}

function select_all_subscriptions() {
  for(i=0; i < document.form_subs_list.elements.length;i++) {
    var el = document.form_subs_list.elements[i];
    if (el.name.indexOf("sub-") == 0) {
      el.checked= true; 
    }
  }
}

function unselect_all_subscriptions() {
  for(i=0; i < document.form_subs_list.elements.length;i++) {
    var el = document.form_subs_list.elements[i];
    if (el.name.indexOf("sub-") == 0) {
      el.checked= false;
    }
  }
}

function orderby(attr_name) {
    document.form_subs_list.order_by.value= attr_name;
    document.form_subs_list.submit();
}

function change_limit_count() {
    var lcount= parseInt(document.form_subs_list.limit_count_list.value);
    document.form_subs_list.limit_count.value= lcount;
    document.form_subs_list.limit_offset.value= 0;
    document.form_subs_list.submit();
}

function goto_page( page_n ) {
    var lcount= parseInt(document.form_subs_list.limit_count.value);
    document.form_subs_list.limit_offset.value= lcount * ( page_n - 1 );
    document.form_subs_list.submit();
}

function prev_page() {
    var loffset= parseInt(document.form_subs_list.limit_offset.value);
    var lcount= parseInt(document.form_subs_list.limit_count.value);
    document.form_subs_list.limit_offset.value= loffset - lcount;
    document.form_subs_list.submit();
}
function next_page() {
    var loffset= parseInt(document.form_subs_list.limit_offset.value);
    var lcount= parseInt(document.form_subs_list.limit_count.value);
    document.form_subs_list.limit_offset.value= loffset + lcount;
    document.form_subs_list.submit();
}
// -->
</script>

<h1>Subscriptions for <?= $user->getUserAttribute(ATTR_COMMONNAME) ?> 
(<?= $user->getUserAttribute(ATTR_USER_UID) ?>)</h1>


<form name="form_subs_list" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="uid" value="<?= $user_id ?>" />
  <input type="hidden" name="action" value="unknown" />
  <input type="hidden" name="limit_offset" value="<?= $limit_offset ?>" />
  <input type="hidden" name="limit_count" value="<?= $limit_count ?>" />
  <input type="hidden" name="order_by" value="<?= $orderby ?>" />
  

<?= $n_subscriptions ?> subscriptions found (<select name="limit_count_list" onChange="change_limit_count()">
<?php
    // drop down list 10, 25, 50, 100, 200
    $limit_counts= array( 10, 25, 50, 100, 200 );
    foreach( $limit_counts as $count ) {
        $selected= "";
        if ($count == $limit_count) $selected= "selected ";
        print "<option value=\"$count\" $selected>$count";
    }
?>
</select> subs/page on <?= $n_pages ?> pages)<br>

 <?php
 if ($n_pages > 1) {
    // show << | 1 | 2 | ... | n | >> control
    if ($page_n > 1) { 
        print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
    }
    else {
        print "&lt;&lt; ";
    }
    foreach( range(1,$n_pages) as $page ) {
        if ($page == $page_n) {
            print "| <b>$page</b> ";
        }
        else {
            print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
        }
    }
    if ($page_n < $n_pages ) {
        print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
    }
    else {
        print "| &gt;&gt;";
    }
 }
 ?>
 <p/>
  
  <table width="95%" border="0" cellspacing="0" class="data-table">

    <!-- ************  table header ********************** -->
    <tr>
      <th class="data" width="1%">&nbsp;</th>
      <th class="data"><a href="javascript:orderby('role_title')" title="Sort by Title">
      <?php if ($orderby == 'role_title') print "<i>Role Title</i>";
            else print "Role Title"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('subscription_state')" title="Sort by Status">
      <?php if ($orderby == 'subscription_state') print "<i>Status</i>";
            else print "Status"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('date_created')" title="Sort by Date">
      <?php if ($orderby == 'date_created') print "<i>Subscribed On</i>";
            else print "Subscribed On"; ?>
      </a></th>
      <th class="data"><a href="javascript:orderby('subscription_last_login')" title="Sort by Date">
      <?php if ($orderby == 'subscription_last_login') print "<i>Last Access</i>";
            else print "Last Access"; ?>
      </a></th>
      <th class="data">Actions</th>
    </tr>  

   <?php if ($n_subscriptions < 1): ?>
  
     <tr>      
      <td width="100%" valign="top" class="data-odd last" colspan="6">
      This user have no subscriptions. See the menu on the left and choose <a href="list-my-users.php">My Users</a> or <a href="list-my-roles.php">My Roles</a>.
      </td>
     </tr>
   
   <?php else: ?>
     <?php
     $i= 0;
     $subs_attrs=& $user->getSubscriptionsAttributes($orderby,$limit_offset,$limit_count);
     foreach($subs_attrs as $sub_attrs):
       $style= ($i++ % 2 == 0) ? "data-even" : "data-odd";
       $rid= $sub_attrs['role_id'];
       $role=& Role::reconstituteById($rid);
       $role_is_mine= $admin->canManageSubscriptions($role);
       $sub_id= "$user_id-$rid";
       $sub_status= $sub_attrs['subscription_state'];
       $sub_date_created= $sub_attrs['date_created'];
       $sub_last_login= $sub_attrs['subscription_last_login'];
       $role_title= $sub_attrs['role_title'];
     ?>
    <tr>
      <!-- ************ checkbox column ************* -->
      <td class="<?= $style ?>">
        <?php if ($role_is_mine): ?>
            <input type="checkbox" name="sub-<?= $sub_id ?>" />
        <?php else: ?>
            &nbsp;
        <?php endif; ?>
      </td>
    
      <!-- ************ role title column ************* -->
      <td class="<?= $style ?>">
      <?= $role_title ?>
      </td>
    
      <!-- ************ sub status ************* -->
      <td class="<?= $style ?>">
      <?php
        switch($sub_status) {
        case  SUBSCRIPTION_STATE_ACCEPTED:
            echo "Accepted"; break;
        case  SUBSCRIPTION_STATE_REJECTED:
            echo "Rejected"; break;
        case  SUBSCRIPTION_STATE_SUSPENDED:          
            echo "Suspended"; break;
        case SUBSCRIPTION_STATE_PENDING:
            echo "Pending"; break;        
        default:
            echo "Unknown ($sub_status)"; break;                  
        }
      ?>
      </td>

      <!-- ************ subscribed on ************* -->
      <td class="<?= $style ?>">
        <?= Formatting::formatMySqlDate($sub_date_created,true) ?>
      </td>

      <!-- ************ last access ************* -->
      <td class="<?= $style ?>">
        <?= Formatting::formatMySqlDate($sub_last_login,true) ?>
      </td>
      
      <!-- ************ actions column ************* -->    
      <td class="<?= $style ?> last">
        <a href="show-subscription-details.php?uid=<?= $user_id ?>&rid=<?= $rid ?>" title="View Subscription Details">Details</a>
      <?php if ($role_is_mine): ?>
        | <a href="javascript:delete_subscription('<?= $sub_id ?>')" title="Delete Subscription">Delete</a>
        <?php if ($sub_status == SUBSCRIPTION_STATE_ACCEPTED): ?>
        | <a href="javascript:suspend_subscription('<?= $sub_id ?>')" title="Suspend Subscription">Suspend</a>
        <?php  elseif ($sub_status == SUBSCRIPTION_STATE_REJECTED): ?>
        | <a href="javascript:accept_subscription('<?= $sub_id ?>')" title="Accept Subscription">Accept</a>
        <?php  elseif ($sub_status == SUBSCRIPTION_STATE_SUSPENDED): ?>
        | <a href="javascript:resume_subscription('<?= $sub_id ?>')" title="Resume Subscription">Resume</a>
        <?php  endif; ?>
      <?php endif; ?>
      </td>    
    </tr>
 
   <?php endforeach; ?>
   

    <!-- *************** last row with summary actions *************** -->
    <tr>
      <td colspan="6" class="data-even">
        <a href="javascript:select_all_subscriptions()">Select All</a> 
        | <a href="javascript:unselect_all_subscriptions()">Unselect All</a>
      </td>
    </tr>
    
    <!-- *************** action buttons *************** -->    
    <tr>
      <td valign="top" class="form-control" align="left" colspan="6">
        <input type="button" name="do_accept" value="Accept Selected" onClick="javascript:accept_selected()"> 
        <input type="button" name="do_delete" value="Delete Selected" onClick="javascript:delete_selected()"> 
        <input type="button" name="do_suspend" value="Suspend Selected" onClick="javascript:suspend_selected()"> 
      </td>
    </tr>    

   </table>   
 </form>
 <?php endif; ?>

<p>

 <?php
 if ($n_pages > 1) {
    // show << | 1 | 2 | ... | n | >> control
    if ($page_n > 1) { 
        print "<a href=\"javascript:prev_page()\" title=\"Previous Page\">&lt;&lt;</a> ";
    }
    else {
        print "&lt;&lt; ";
    }
    foreach( range(1,$n_pages) as $page ) {
        if ($page == $page_n) {
            print "| <b>$page</b> ";
        }
        else {
            print "| <a href=\"javascript:goto_page($page)\" title=\"Go to Page $page\">$page</a> ";
        }
    }
    if ($page_n < $n_pages ) {
        print "| <a href=\"javascript:next_page()\" title=\"Next Page\">&gt;&gt;</a>";
    }
    else {
        print "| &gt;&gt;";
    }
 }
 ?>

 <p/> 

<?php
include("include/adminFooter.inc");
?>
