<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004  SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: login.php,v 1.9 2006/09/26 12:27:28 vlry Exp $
 */
/* ****************************************************************** */

/**
* This page displays an authentication form and checks whether
 *   the current user can be authenticated as Local SNA-Portal
 *   Administratos.
*
* <b>Request Parameters</b>
* <ul>
*   <li>username=<username>: OPTIONAL. The username 
*    </li>
*   <li>password=<password>: OPTIONAL. The password 
*    </li>
*   <li>do_auth=on: OPTIONAL. If present, performs the authentication. If
*      absent, just displays the input form.
*    </li>
*
*  </ul>
*
* 
* @version $Revision: 1.9 $
* @package AdminPortal
*/

require_once("include/application.inc");
$logger=& Logger::getInstance();


/** 
 * Checks the request parameters 
 *
 * @param array  &$errors   an OUT-parameter with an array of error message
 *              The following keys are defined:
 *              <ul>
 *                <li>'username'  -> the error message for input supplied
 *                    in $_REQUEST["username"]
 *                <li>'password'  -> the error message for input supplied
 *                    in $_REQUEST["password"]
 *              </ul>
 * 
 * @return boolean  true, if input is accepted; false, otherwise
 *
 * @access private
 */
function check_request_parameter(&$errors) {
  $ret = true;

  // make sure we have a non-empty username
  //
  if (empty($_REQUEST["username"])) {
    $errors["username"] = "Username must not be empty.";
    $ret = false;
  }

  // make sure we have a non-empty password
  if (empty($_REQUEST["password"])) {
    $errors["password"] = "Password must not be empty.";
    $ret = false; 
  }
  return $ret;
}

/**
 * Authenticates a administrator against the SNA-Portal user database
 *
 * @param string $username    the username
 * @param string $password    the password
 * @param array  &$errors     an OUT-parameter with an array of error message
 *               The following keys are defined:
 *                'auth-error'  -> the error message for an authentication
 *                    error
 *
 * 
 * @return boolean true, if the administrator could be authenticated; false,
 *   otherwise
 *
 * @access private
 */
function do_authenticate($username, $password, &$errors) {
  $logger=& Logger::getInstance();
  // create a administrator object with the given user name (uid)
  $admin =& Administrator::reconstituteByUid($username);
  if (is_null($admin)) {
     $errors["auth-error"] = "User $username does not have enough privileges to enter as Administrator";
     $logger->error("Local login: admin $username is not administrator");
     return false;
  }
  if (! $admin->checkPassword($password)) {
     $errors["auth-error"] = "Invalid credential. Please try again.";
     $logger->error("Local login: admin $username wrong password");
     return false; 
  }

  $logger->debug("Local login: admin $username authenticated");
  return true; 
}




//
// form processing
//
if (isset($_REQUEST["do_auth"])) {
  $errors = array();

  if (check_request_parameter(&$errors)) {
    $username = $_REQUEST["username"];
    $password = $_REQUEST["password"];
    $username= trim($username);
    $password= trim($password);
    if (do_authenticate($username, $password, &$errors)) {

       $admin =& Administrator::reconstituteByUid($username);
       Session::create();
       Session::init($admin);
      
       $logger->info("Local login: admin $username login");
       Event::storeAuthEvent("Local admin login: $username",$admin->getID());
       $admin->updateLoginDate();

       // redirect to the start page  
       header("Location: " . PORTAL_BASE_URL . "/admin/start-page.php");
       exit;
    }
    else {
        Event::storeAuthEvent("Local admin wrong username or password: $username",
                              1, EVENT_LEVEL_WARNING);
    }
  }
}


//
// HTML content
//
require_once("lookandfeel/current/lf-specific-functions.inc");

?>
<html>
<head>
<link rel="stylesheet" href="<?= lf_get_user_style_sheet_url() ?>" type="text/css">
<style type="text/css">
<!--
    /* WISEBEDaai Login pages style */
    img.non-aai-login-button {
        padding-bottom: 8; 
        padding-top: 40;
    }
    input.non-aai-login-button {
	   padding-bottom: 8; 
	   padding-top: 12; 
	   float: right;
	}
    
    /* boxing styles */
    td.wisebedaai {
      font-family: Verdana, Arial, Helvetica, sans-serif; 
      background: #FFFFFF; 
      color: #000000; 
      font-size: 12;
    }
    td.top-left-x {
        background-image: url('../lookandfeel/current/obenlinksx.gif');
        height: 14px;
        width: 14px;
    }
    td.top-middle-x {
        background-image: url('../lookandfeel/current/obenmittex.gif');
        height: 14px;
    }
    td.top-right-x {
        background-image: url('../lookandfeel/current/obenrechtsx.gif');
        height: 14px;
        width: 14px;
    }
    td.middle-left-x {
        background-image: url('../lookandfeel/current/mittelinksx.gif');
        width: 14px;
    }
    td.middle-right-x {
        background-image: url('../lookandfeel/current/mitterechtsx.gif');
        width: 14px;
    }
    td.bottom-left-x {
        background-image: url('../lookandfeel/current/untenlinksx.gif');
        height: 14px;
        width: 14px;
    }
    td.bottom-middle-x {
        background-image: url('../lookandfeel/current/untenmittex.gif');
        height: 14px;
    }
    td.bottom-right-x {
        background-image: url('../lookandfeel/current/untenrechtsx.gif');
        height: 14px;
        width: 14px;
    }
-->
</style>
</head>
<body>

<p class="welcome-title" style="padding-top: 1cm; text-align: center;">Welcome to the SNA-Portal</p>


<p style="margin-left: 1cm; margin-right: 1cm; text-align: center">
Using the form below you can login as Portal, Role or Subscription Administrator
if you received a local SNA-Portal username and password.
</p>
<p style="margin-left: 1cm; margin-right: 1cm; text-align: center">
Please <a href="aai/login.php">click here</a> if you want to authenticate with your AAI Home Organization.
</p>

<!-- local login box -->
<div align="center">
  <table width="300" border="0" cellpadding="0" cellspacing="0">
    <tr>
      <td class="top-left-x"></td>
      <td class="top-middle-x"></td>
      <td class="top-right-x"></td>
    </tr>
     <!-- links + content + right -->
    <tr>
       <!-- links border -->
      <td class="middle-left-x"></td>
       <!-- content -->
      <td class="wisebedaai" valign="top">
         <p><b>Local login for Administrators</b></p>
         <?php if (isset($errors["auth-error"])): ?>
            <span class="error-message"><?= $errors["auth-error"] ?></span>
            <p/>
         <?php endif; ?>
         <form method="post" action="<?= $_SERVER['PHP_SELF'] ?>">
         <input type="hidden" name="do_auth" value="1" />
         <table width="90%">
         <!-- username -->
         <tr>
           <td class="wisebedaai">Username</td>
           <td class="wisebedaai"><input type="text" name="username" size="20" maxlength="30" tabindex="1">
           <?php if (isset($errors["username"])):  ?>
               <br/><span class="error-message"><?= $errors["username"] ?></span>
           <?php endif; ?>
           </td>
         </tr>
         <!-- password -->
         <tr>
           <td class="wisebedaai">Password</td>
           <td class="wisebedaai"><input type="password" name="password" size="20" maxlength="30" tabindex="2">
           <?php if (isset($errors["password"])):  ?>
               <br/><span class="error-message"><?= $errors["password"] ?></span>
           <?php endif; ?>
           </td>
         </tr>
         </table>
         <input type="image" src="../lookandfeel/current/button-login-gray.gif" width="52" height="19" border="0" class="non-aai-login-button" alt="login" align="bottom">
         </form>
      </td>
       <!-- right border -->
      <td class="middle-right-x"></td>
    </tr>
    <tr>
      <td class="bottom-left-x"></td>
      <td class="bottom-middle-x"></td>
      <td class="bottom-right-x"></td>
    </tr>
  </table>
</div>

</body>
</html>





