<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: new-invitation.php,v 1.49 2007/10/10 15:35:26 vlry Exp $
 */

//
// local functions
//

/**
 * Extracts one email address from a string.
 *
 * @param $text The string to match.
 * @return String The extracted email or NULL.
 */
function extract_email( $text ) {
    if(eregi("([_a-z0-9-]+(\.[_a-z0-9-]+)*@[_a-z0-9-]+(\.[_a-z0-9-]+)*\.[a-z]{2,3})", $text, $regs)) {
      return $regs[1];
    }
    else {
      return NULL;
    }
}


/**
 * Sends a invitation for a particular role.
 *
 * new-invitation.php?rid=<role_id>[&to=<user_id>]
 *
 * @param rid The Role ID
 * @param to  A User ID to send to [OPTIONAL]
 */

//
// bootstrap application
//
require_once("include/application.inc");

// check for param rid
if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("Request parameter 'rid' is missing");	
}
$rid= $_REQUEST["rid"];

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

// role
$role =& Role::reconstituteById($rid);
if (is_null($role)) {
    ErrorHandling::fatalErrorAndExit("Illegal role id ($rid) submitted");
}

//
// optional request parameter: to=<user_id>
//
if(isset($_REQUEST['invitation_emails'])) {
    $invitation_emails= $_REQUEST['invitation_emails'];   
    //XXX echo "XXX: req.invitation_emails=$invitation_emails<br>";

}

if(isset($_REQUEST['to'])) {

    $to_user_id= $_REQUEST['to'];

    //XXX echo "XXX: req.to=$to_user_id<br>";

    $to_user= User::reconstituteById($to_user_id);
    if ($to_user) {
        $to_name= $to_user->getNameOrUid();
        $to_email= $to_user->getAttribute(ATID_MAIL);
        $invitation_emails= "$to_name &lt;$to_email&gt;";
    }
}

//
// optional request parameter: locale=<locale>
//
if (isset($_REQUEST['do_change_locale'])) {
    $locale= $_REQUEST['locale'];
    
    //XXX echo "XXX: req.do_change_locale->req.locale=$locale<br>";
    
    $locale= strtolower($locale);
    if ( I18N::isSupportedLocale( $locale )) {
        $admin->storeAttribute(ATID_PREFERREDLANGUAGE, $locale); 
        Session::setLocale( $admin );
    }
}


//
// form validation and processing
//
$input_errors= array();
$processing_errors= array();
if (isset($_REQUEST['do_send'])) {

    // get form parameters
    $invitation_from= $_REQUEST['invitation_from'];    
    // read globally
    //$invitation_emails= $_REQUEST['invitation_emails'];    
    $invitation_subject= $_REQUEST['invitation_subject'];
    $invitation_text= $_REQUEST['invitation_text'];
    if (get_magic_quotes_gpc()) {
        $invitation_from= stripslashes($invitation_from);
        $invitation_emails= stripslashes($invitation_emails);
        $invitation_subject= stripslashes($invitation_subject);
        $invitation_text= stripslashes($invitation_text);
    }
    $invitation_from= trim($invitation_from);
    $invitation_emails= trim($invitation_emails);
    $invitation_subject= trim($invitation_subject);
    $invitation_text= trim($invitation_text);
        
    // validate input
    $valid_input= true;
    if (empty($invitation_from)) {
        $input_errors['invitation_from']= "Can't be empty. Please define your Email in your profile.";
        $valid_input= false;
    }
    $from= extract_email($invitation_from);
    if (!$from) { 
        $input_errors['invitation_from']= "No valid Sender email found. Please define your Email in your profile.";
        $valid_input= false;
    }
    if (empty($invitation_subject)) {
        $input_errors['invitation_subject']= "Subject can't be empty.";
        $valid_input= false;
    }
    if (empty($invitation_text)) {
        $input_errors['invitation_text']= "Please enter the email content text.";
        $valid_input= false;
    }


    // process valid and invalid emails
    $valid_emails= array();
    $emails= explode(',',$invitation_emails);
    foreach($emails as $email) {
        $email= trim($email);
        if (!empty($email)) {
            $to= extract_email($email);
            if ($to) {
                $valid_emails[]= $to;
            }
            else {
                $error= "Invalid email address: $email";
                $processing_errors['error_emails'][]= $error;
            }
        }
    }
    
    // validate input emails
    if (empty($valid_emails)) {
        $input_errors['invitation_emails']= "No valid email found in Email List.";
        $valid_input= false;
    }
    
    
    if ($valid_input) {
    
        $invitations_sent= array();
        $rkey= $role->getRoleKey();
        
        // 1. create invitation token for all valid emails 
        //    not already subscribed to the role.
        // 2. send email
        // 3. if no error presist invitation in DB    

        // get all existing accepted subsciptions
        $subscriptions= $role->getSubscriptions(SUBSCRIPTION_STATE_ACCEPTED);

        foreach($valid_emails as $to) {

            // checks if a user with this email is already registered
            $user_registered= false;
            foreach($subscriptions as $subscription) {
                $user= $subscription->getUser();
                $user_email= $user->getUserAttribute(ATTR_EMAIL);
                if ( strcasecmp($to, $user_email) == 0) {
                    $user_registered= true;
                    $error= "User $to is already subscribed to this role (no invitation sent)";
                    $processing_errors['error_invitations'][]= $error;
                }
            }
            if ($user_registered) {
                // user is already registered for the role, skip him
                continue;
            }
    
            // create new invitation
            $invitation= new Invitation($to,$rid);
            $token= $invitation->getToken();
    
            // email datas
            $subject= $invitation_subject;

            // get text body and replace INVITATION_URL and LOGIN_URL
            $invitation_url= PORTAL_BASE_URL . "/user/aai/invitation.php?rid=$rkey&token=$token";
            $login_url= PORTAL_BASE_URL . "/user/aai/login.php?rid=$rkey";

            $body= $invitation_text;
            $body= str_replace('@-INVITATION_URL-@', $invitation_url, $body);
            $body= str_replace('@-LOGIN_URL-@', $login_url, $body);
            
            
            // send invitation
            $message= new Messaging(); 
            $admin_attr= $admin->getUserAttributes(ATTR_COMMONNAME);
            $from_name= "Admin: {$admin_attr[ATTR_COMMONNAME]}";
            $sent= $message->sendAsMail($from,$to,$subject,$body, false, $from_name);
        
            if ($sent) {
                $invitations_sent[]= $to;

                // store in DB
                $admin_id= $admin->getID();
                $invitation->setCreatorUserId( $admin_id );
                $stored= $invitation->persist();
                if (!$stored) {
                    $error= "User $to already received an Invitation for this role, but did not use it (invitation resent)";
                    $processing_errors['error_invitations'][]= $error;
                }
            }
            else {
                $error= "Failed to send email from: $from to: $to, " . $message->getError();
                $processing_errors['error_messages'][]= $error;
            }
        
        } // foreach emails
        
        //
        // HTML confirmation page (with errors)
        //
        include('include/adminHeader.inc');
        ?>
        <p align="center">
        <span class="button-reference"><a href="list-invitations.php?rid=<?= $rid ?>">List Unused Invitations</a></span>
        </p>        
        <h1>Invitation Confirmations</h1>
        <p><strong><?= count($invitations_sent) ?> invitations were sent</strong>:

        <?php
        if (count($invitations_sent)) {
            foreach($invitations_sent as $sent_to) {
                echo "<br>$sent_to";                        
            }
        }
        
        // TODO: list sent emails?
        
        // infos then errors
        if ( isset($processing_errors['error_invitations']) and !empty($processing_errors['error_invitations'])) {
            echo "<p><strong>Invitations Info:</strong>";
            foreach($processing_errors['error_invitations'] as $error_invitations) {
                echo "<br>$error_invitations";            
            }
        }
        if ( isset($processing_errors['error_emails']) and !empty($processing_errors['error_emails'])) {
            echo "<p><strong>Email Address Errors:</strong>";
            foreach($processing_errors['error_emails'] as $error_emails) {
                echo "<br>$error_emails";            
            }
        }
        if ( isset($processing_errors['error_messages']) and !empty($processing_errors['error_messages'])) {
            echo "<p><strong>Email Delivery Errors:</strong>";
            foreach($processing_errors['error_messages'] as $error_messages) {
                echo "<br>$error_messages";            
            }
        }
        
        include("include/adminFooter.inc");
        exit;
        
        
    } // 

} // process form

//
// init I18N for admin
I18N::initLocale();
I18N::loadMessageBundle("notification/invitation");


//
// start HTML content
//
include("include/adminHeader.inc");
// 
// vars used in messages and texts
$role_title= $role->getTitle();
$role_key= $role->getRoleKey();
$admin_name= $admin->getNameOrUid();
$admin_email= $admin->getAttribute(ATID_MAIL);

?>

<p align="center">
  <span class="button-reference"><a href="list-invitations.php?rid=<?= $rid ?>">List Unused Invitations</a></span>
</p>

<h1>Send new Invitations for Role: <?= $role_title ?> (<?= $role_key ?>)</h1>

<p>
Invitations allows you to automatically subscribe user to a role. 
Each user will receive an email with a one time registration token.
</p>

<p>
First choose the language, then you can enter email addresses and modify the default text.
</p>


<!-- main table -->
<table width="90%" border="0" cellspacing="0" class="form-table">

<form name="invitationForm" action="<?= $_SERVER['PHP_SELF'] ?>" method="post">
<input type="hidden" name="rid" value="<?= $rid ?>"/>

<!-- language locale -->
<tr>
    <td width="20%" valign="top" class="form-label form-first">
    Language
    </td>
	<td valign="top" class="form-entry form-first">
    <select name="locale" >
    <?php
    $languages= I18N::getSupportedLocales();
    foreach($languages as $langid => $langdisplayname): 
       $selected= (I18N::getLocale() == $langid) ? "SELECTED" : ""; ?>
       <option value="<?= $langid ?>" <?= $selected ?> ><?= $langdisplayname ?></option> 
    <?php endforeach; ?>
    </select>
    <input type="submit" name="do_change_locale" value="Change" />
    </td>
</tr>    

<!-- Invitation form -->
<tr>
    <!-- Invitation Sender -->
    <td width="20%" valign="top" class="form-label form-first">
    Sender
    </td>
    <td valign="top" class="form-entry form-first">
    <?php 
        $invitation_from= "$admin_name &lt;$admin_email&gt;";
    ?>
    <tt><?= $invitation_from ?></tt>
    <?php  if (isset($input_errors['invitation_from'])):  ?>
      <br/><span class="error-message"><?= $input_errors['invitation_from'] ?></span>
    <?php  endif; ?>
    <input type="hidden" name="invitation_from" size="60" value="<?= $invitation_from ?>"/>
    </td>
</tr>
<tr>
    <!-- Email list -->
    <td width="20%" valign="top" class="form-label">
    Email List
    </td>
    <td valign="top" class="form-entry">
    <p>Enter the list of email you want to send invitation to. Please enter the emails
    as a comma separated list (CSV).
    </p>
    <?php if( !isset($invitation_emails) ) { $invitation_emails= ''; } ?>
    <textarea name="invitation_emails" cols="60" rows="10"><?= $invitation_emails ?></textarea>
    <?php  if (isset($input_errors['invitation_emails'])):  ?>
      <br/><span class="error-message"><?= $input_errors['invitation_emails'] ?></span>
    <?php  endif; ?>

    </td>
</tr>
<tr>
    <!-- Invitation Subject -->
    <td width="20%" valign="top" class="form-label">
    Invitation Subject
    </td>
    <td valign="top" class="form-entry">

    <?php if( !isset($invitation_subject) ) { 
        $invitation_subject= I18N::getMessage('invitation.subject', $role_title );
    } ?>
    <input type="text" name="invitation_subject" size="60" value="<?= $invitation_subject ?>"/>
    <?php  if (isset($input_errors['invitation_subject'])):  ?>
      <br/><span class="error-message"><?= $input_errors['invitation_subject'] ?></span>
    <?php  endif; ?>
    </td>
</tr>
<tr>
    <!-- Invitation Text -->
    <td width="20%" valign="top" class="form-label">
    Invitation Text
    </td>
    <td valign="top" class="form-entry">
    <p>Here is the default invitation text.<br>The keywords <tt><b>@-INVITATION_URL-@</b></tt> and <tt><b>@-LOGIN_URL-@</b></tt>
    in the text will be automatically replaced with the corresponding one-time invitation URL,
    respectively the login URL.
    </p>
    <?php if( !isset($invitation_text) ) { 
        // build the text from I18N notification/invitation.messages
        $invitation_text= I18N::getMessage('invitation.text_header'); 
        $invitation_text.= I18N::getMessage('invitation.text_registrar', $admin_name, $admin_email, $role_title); 
        $invitation_text.= I18N::getMessage('invitation.text_urls'); 
        $invitation_text.= I18N::getMessage('invitation.text_instructions', $role_title); 
        $invitation_text.= I18N::getMessage('invitation.text_support', $admin_name, $admin_email, $role_title); 
        // if webct?
        $role_adapter= $role->getRoleAdapter();
        if (strpos($role_adapter,'webct') !== false ) {
            $invitation_text.= I18N::getMessage('invitation.text_webct'); 
        }
        $invitation_text.= I18N::getMessage('invitation.text_footer', $role_title, $admin_name); 
    } ?>
    <textarea name="invitation_text" cols="70" rows="30" wrap="soft"><?= $invitation_text ?></textarea>
    <?php  if (isset($input_errors['invitation_text'])):  ?>
      <br/><span class="error-message"><?= $input_errors['invitation_text'] ?></span>
    <?php  endif; ?>
    </td>
</tr>
<tr>
    <!-- Cancel and Send -->
    <td width="100%" valign="top" align="center" class="form-control" colspan="2">
    <input type="submit" name="do_send" value="Send Invitation" />
    </td>    
</tr>
  
</tr>
</table>
</form>

<?php 
include("include/adminFooter.inc");
?>
