<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: show-subscription-details.php,v 1.27 2005/11/28 13:20:15 vlry Exp $
 */

/**
* This page displays detail information about a subscription
*
* <b>Request Parameters</b>
* <ul>
*   <li>uid=<uid>: MANDATORY. The internal user ID of the owner of
*     the subscription. <uid> must be
*    a valid integer > 0.
*    </li>
*   <li>rid=<rid>: MANDATORY. The internal role ID of the role 
*      which the subscription exists for. <rid> must be
*    a valid integer > 0.
*    </li>
*  </ul>
*
* 
* @version $Revision: 1.27 $
* @package AdminPortal
*/


/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */

/**
 * Ensures that all expected
 *   request parameters are actually passed in and that they have
 *   the acceptable values
 *  Aborts with exit() if an error is detected. 
 *
 * @access private
 */ 
function ensure_request_parameters_valid() {

  // make sure mandatory uid is set
  //
  if (! isset($_REQUEST["uid"])) {
     ErrorHandling::fatalErrorAndExit("request parameter 'uid' is missing");
  }

  // make sure the uid is numeric and positive
  //
  if (! is_numeric($_REQUEST["uid"]) || $_REQUEST["uid"] < 0) {
     ErrorHandling::fatalErrorAndExit("illegal value '". $_REQUEST["uid"] . "' for request parameter 'uid'");
  }   


  // make sure mandatory rid is set
  //
  if (! isset($_REQUEST["rid"])) {
     ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing");
  }

  // make sure the uid is numeric and positive
  //
  if (! is_numeric($_REQUEST["rid"]) || $_REQUEST["rid"] < 0) {
     ErrorHandling::fatalErrorAndExit("illegal value '". $_REQUEST["rid"] . "' for request parameter 'rid'");
  }
}



// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("current user is not priviledged to enter this page"); 
}
Session::init($admin);

//  load the user object
//
$uid= $_REQUEST["uid"];
$user=& User::reconstituteById($uid);
if (is_null($user)) {
  ErrorHandling::fatalErrorAndExit("no user with user ID '$uid' found");
}

//  load the role object
//
$rid= $_REQUEST["rid"];
$role =& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("no role with user ID '$rid' found");
}
$role_is_mine= $admin->canManageSubscriptions($role);

$subscription =& Subscription::reconstitute(&$user, &$role);
if (is_null($subscription)) {
  ErrorHandling::fatalErrorAndExit("no subscription for user '$uid' and role '$rid' found"); 
}
$sub_id= "$uid-$rid";

/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

<h1>Subscription Details</h1>
  
  This page displays detailed information about a subscription to one
  of your roles.  
  <p/>
  <table width="90%" border="0" cellspacing="0" class="form-table">

    <tr>
      <td width="30%" valign="top" class="form-label form-first">
        Role Title      
      </td>
      <td width="60%" valign="top" class="form-entry form-first">
       <?= $role->getTitle() ?>
       <?php if ($admin->canEditRole($role)): ?>
       &nbsp;&nbsp;<a href="edit-role.php?rid=<?= $rid  ?>">Edit</a>
       <?php endif; ?>
      </td>
    </tr>
    <tr>
      <td width="30%" valign="top" class="form-label">
        Role Key
      </td>
      <td width="60%" valign="top" class="form-entry">
       <?= $role->getRoleKey() ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label form-first">
        User (User ID)
      </td>
      <td width="60%" valign="top" class="form-entry form-first">
       <?= $user->getNameOrUid() ?> (<?= $user->getUID() ?>)
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
        Last Access
      </td>
      <td width="60%" valign="top" class="form-entry">
       <?= Formatting::formatMysqlDate($subscription->getLastLogin(), true) ?>
      </td>
    </tr>
    
    <tr>
      <td width="30%" valign="top" class="form-label form-first">
        Subscribed On
      </td>
      <td width="60%" valign="top" class="form-entry form-first">
       <?= Formatting::formatMySQLDate($subscription->getDateCreated()) ?>
      </td>
    </tr>
    
    
    <tr>
      <td width="30%" valign="top" class="form-label">
        Subscription Status
      </td>
      <td width="60%" valign="top" class="form-entry">
       <?php 
         $state = $subscription->getState();
       ?>
      <?php if ($state == SUBSCRIPTION_STATE_ACCEPTED): ?>
      Accepted
      <?php if ($role_is_mine): ?>
      &nbsp;&nbsp;
      <a href="list-subscriptions.php?rid=<?= $rid ?>&action=do_suspend&sub-<?= $sub_id ?>=on">Suspend</a>
      <?php endif; ?>
      <?php elseif ($state == SUBSCRIPTION_STATE_REJECTED): ?>
      Rejected 
      <?php if ($role_is_mine): ?>
      &nbsp;&nbsp;
      <a href="list-subscriptions.php?rid=<?= $rid ?>&action=do_accept&sub-<?= $sub_id ?>=on">Accept</a>       
      | <a href="new-invitation.php?rid=<?= $rid ?>&to=<?= $uid ?>">Invite</a>
      <?php endif; ?>
      <?php elseif ($state == SUBSCRIPTION_STATE_SUSPENDED): ?>
      Suspended
        <?php if ($role_is_mine): ?>
        &nbsp;&nbsp;
        <a href="list-subscriptions.php?rid=<?= $rid ?>&action=do_resume&sub-<?= $sub_id ?>=on">Resume</a>     
        <?php endif; ?>
      <?php elseif ($state == SUBSCRIPTION_STATE_PENDING): ?>
      Pending
        <?php if ($role_is_mine): ?>
        &nbsp;&nbsp;
        <a href="list-subscriptions.php?rid=<?= $rid ?>&action=do_accept&sub-<?= $sub_id ?>=on">Accept</a> 
        <?php endif; ?>
      <?php endif; ?>
      <?php if ($role_is_mine): ?>
      | <a href="list-subscriptions.php?rid=<?= $rid ?>&action=do_delete&sub-<?= $sub_id ?>=on">Delete</a>
      <?php endif; ?>
      </td>
    </tr>

  </table>
  
  
  <!-- ******************* Required Attributes ******************* -->
   
   <p/>
  According to your Attribute Request Policy for this role the
  subscriber provided the following information when he submitted
  the subscription request.
  <p/>
  <strong>Trusted</strong> information was supplied by the AAI Home Organization of subscriber. <strong>Untrusted</strong> information was supplied by the subscriber himself. 
  <p/>
  <table width="90%" border="0" cellspacing="0" class="form-table">

  <tr>
      <th width="30%" valign="top" class="data">
      Name
      </th>
      <th width="50%" valign="top" class="data">
      Value
      </th>
      <th width="20%" valign="top" class="data">
      Trust Level
      </th>
  </tr>
 
   <tr>
      <td width="30%" valign="top" class="data-even">
      User ID
      </td>
      <td width="50%" valign="top" class="data-even">
       <?= $user->getUid() ?>
      </td>
      <td width="20%" valign="top" class="data-even last">
        Trusted 
      </td>
    </tr>
 
  <?php
   $attributes = $subscription->getSubscriptionAttributes();
   $i=1;
   ?>
   <?php foreach ($attributes as $attr): ?>      
       <?php
        $value = $attr["attribute_value"];
        $name = AttributeType::getAttributeTypeName((int)$attr["attribute_type"]);
        $isTrusted = $attr["attribute_trusted"];
        $style = ($i++ % 2 == 0) ? "data-even" : "data-odd";
        ?>

         <tr>
            <td width="30%" valign="top" class="<?= $style ?>">
                <?= $name ?>
            </td>
            <td width="50%" valign="top" class="<?= $style ?>">
                 <?= Formatting::formatTableCellContent($value) ?>
            </td>
            <td width="20%" valign="top" class="<?= $style ?> last">
               <?php if ($isTrusted) : ?>
               Trusted
               <?php else: ?>
               Untrusted
               <?php endif; ?>
            </td>

          </tr>

   <?php endforeach; ?>
  </table>
  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
