<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2006 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: login.php,v 1.27 2007/03/07 14:53:02 vlry Exp $
 */


/**
* This page is the Shibboleth protected start page for the user part of
* the SNA-Portal. After a user authenticates at its Home Organization (HO)
* and after the local Shibboleth Attribute Requester (SHAR) retrieved
* its attributes from the Home Organization, this page is invoked.
*
* This page opens a session and creates a new user according to the 
* AAI User ID supplied in the HTTP header "Shib-WISEBEDPerson-personUniqueID".
* If the user already exists, if just updates the local database with the
* current attribute values supplied by AAI.
*
* @version $Revision: 1.27 $
* @package UserPortal
* @subpackage AAI
*/

require_once("include/application.inc");
$logger=& Logger::getInstance();

//
// get unique id (AAI)
//
if (!isset($_SERVER['Shib-WISEBEDPerson-personUniqueID']) or empty($_SERVER['Shib-WISEBEDPerson-personUniqueID'])) {
    // no unique id from shibboleth
    ErrorHandling::fatalErrorAndExit("AAI Error: HTTP header Shib-WISEBEDPerson-personUniqueID missing or empty.");
}
$aai_uniqueid= $_SERVER['Shib-WISEBEDPerson-personUniqueID'];

//
// reconstitute an existing user or create a new one
// 
$user=& User::reconstituteByUid($aai_uniqueid);
// if there is no such user yet, create one
if (is_null($user)) {
  $user=& User::create($aai_uniqueid);
  if (is_null($user)) {
    Event::storeSystemEvent("Failed to create user: $aai_uniqueid", EVENT_LEVEL_ERROR);
    ErrorHandling::fatalErrorAndExit("Failed to create new AAI user ($aai_uniqueid).");
  }
  $logger->info("new AAI user $aai_uniqueid created.");
  Event::storeUserEvent("AAI user created: $aai_uniqueid", $user->getID());
}

//
// create the session and init the user session
//
Session::create();
Session::init($user);

$logger->info("AAI login: user $aai_uniqueid login");
Event::storeAuthEvent("AAI user login: $aai_uniqueid", $user->getID());
$user->updateLoginDate();

//
// redirect to requested role (login.php?rid=<role_key>)
// or default user start page.
//
$location= PORTAL_BASE_URL . '/user/start-page.php';
if ( isset($_REQUEST['rid']) ) {
    $rkey= $_REQUEST['rid'];
    $role=& Role::reconstituteByKey($rkey);
    if (is_null($role)) {
        Event::storeAccessEvent("Invalid role: $rkey for user: $aai_uniqueid",
                                $user->getID(),EVENT_LEVEL_ERROR);
        ErrorHandling::fatalErrorAndExit(
            "Invalid Role ($rkey).",
            "<p>The System was not able to give you access to the role  because:"
           ."<ul><li>the URL is invalid or wrong (e.g. truncated, contains blanks,...).<br>"
           ."Try to correct the URL (remove space %20, edit the bookmark, copy/paste the original, ...)</li>"
           ."<li>If you received an Invitation by email, try to access the role by clicking on the URL annotated in the Invitation."
           ."</li></ul></p>"
        );
    }
    // only AAI login can force a subscription to a password protected role
    if ($role->isPasswordProtected()) {
        // lookup subscription and create if don't exist
        $uid= $user->getID();
        $rid= $role->getID();
        if ( !Subscription::exists($uid,$rid) ) {
            Event::storeSubscriptionEvent("Password subscription created: $uid -> $rid", $uid);
            $subscription=& Subscription::create($user, $role);
            $subscription->accept();
        }
    }
    $role_id= $role->getID();
    $location= PORTAL_BASE_URL . "/user/go.php?rid=$role_id";
    // optional req param courseid for WebCT adapters
    if ( isset($_REQUEST['courseid']) ) {
        $courseid= $_REQUEST['courseid'];
        $location= "$location&courseid=$courseid";
    }
}

$logger->debug("AAI login: redirect: $location");
header("Location: $location");

?>
